/** 
 *******************************************************************************
 * @file    main.c
 * @brief   ADC Monitor Sample Application. \n
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TSB_CMSIS_CORE.h"  /*!< Group Header file. */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "adc.h"
#include "cg.h"
#include "t32a.h"
#include "bsp.h"
#include "bsp_adc.h"
#include "bsp_led.h"
#include "uart_io.h"

/**
 *  @addtogroup Example Example
 *  @{
 */

/** 
 *  @addtogroup ADC_Monitor ADC_Monitor
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup ADC_Monitor_config ADC_Monitor Config
 *  @{
 */

/** 
 *  @addtogroup CountConfiguration Time Configuration
 *  @brief    Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_OUTPUT_COUNT_TIME           ((uint32_t)(5000))
#define CFG_ADC_COUNT_TIME              ((uint32_t)(1000))

#define CFG_ADC_CMP_25_SMALL_THAN       (0x00007F60UL)      /*! If temp >= 25 c, Small than this value */
#define CFG_ADC_CMP_35_BIG_THAN         (0x00006160UL)      /*! If temp < 36 c,  Big than this value */

/** 
 *  @}
 */ /* End of group CountConfiguration */

/** 
 *  @}
 */ /* End of group ADC_Monitor_config */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup ADC_Monitor_Private_variables ADC_Monitor Private Variables
 *  @{
 */
static volatile uint32_t output_flag = 0;           /*!< Output flag.        */
static volatile uint32_t fIntADC = 0;               /*!< ADC flag.           */
static volatile uint32_t cmp0_flag = 0;             /*!< CMP0 flag.          */
static volatile uint32_t cmp1_flag = 0;             /*!< CMP1 flag.          */
volatile uint32_t data0 = 0;
static uint32_t counter = CFG_OUTPUT_COUNT_TIME;    /* Blink Time Counter    */
static uint32_t adc_counter = CFG_ADC_COUNT_TIME;   /* */
static int32_t degC;                                /*!< degC.          */
/**
 *  @}
 */ /* End of group ADC_Monitor_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup ADC_Monitor_Private_fuctions ADC_Monitor Private Fuctions
 *  @{
 */

/**
 *  @}
 */ /* End of group ADC_Monitor_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup ADC_Monitor_Exported_functions ADC_Monitor Exported Functions
 *  @{
 */
 
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{

    /*------------------------*/
    /* Port Setting           */
    /*------------------------*/
    /* Enable the peripheral clock of ADC unit C */
    GPIO_ADC_C_enable_fsys();
    /* Configure ADC unit C  */
    GPIO_ADC_C_Config();
    /* Enable the peripheral clock of LED */
    GPIO_LED_enable_fsys();
    /* Configure LED  */
    GPIO_LED_Config();

    /*------------------------*/
    /* UART Setting            */
    /*------------------------*/
    uart_io_initialize();

    /*------------------------*/
    /* ADC Setting            */
    /*------------------------*/
    /* Enable the peripheral clock of ADC Unit C */
    ADC_C_enable_fsys();
    /* ADC initial setting */
    ADC_C_Config();
    /* ADC CMP initial setting */
    ADC_C_CP_Config(CFG_ADC_CMP_25_SMALL_THAN , CFG_ADC_CMP_35_BIG_THAN);
    Set_ADC_C_TSET0_value();
    /* interrupt setting */
    NVIC_EnableIRQ(INTADCCP0_IRQn);
    NVIC_EnableIRQ(INTADCCP1_IRQn);

    /*------------------------*/
    /* T32A Interval TimerSetting */
    /*------------------------*/
    /* Enable the peripheral clock of T32A00 */
    T32A_CH0_enable_fsys();
    /* T32A Interval Timer Setting */
    T32A0_A_IntervalTimer_Config();
    /* Interrupt Enable */
    NVIC_EnableIRQ(INTT32A00AC_IRQn);   /* T32A ch.0 TimmerA compare match detection */

    /* Timer Start */
    T32A0_A_Start();
    /*------------------------*/
    /*          Loop          */
    /*------------------------*/ 
    while(1){
        if(fIntADC == 1)
        {
            fIntADC = 0;
            /* ADxREG0 [AIN0] AD Result Value Get */
            data0 = Get_ADC_C_REG0_CP_value();
        }
        if(output_flag == 1)
        {
            degC = bsp_get_thermistors_degC(data0);
            printf("Convert Result Value[THERMISTOR]:0x%04x (%d degrees)\n", data0, degC);

            /* LED output */
            if((cmp0_flag == 0) && (cmp1_flag == 1))       /* degC < tempA */
            {
                GPIO_LED1_OFF();
                GPIO_LED2_OFF();
            }
            else if((cmp0_flag == 1) && (cmp1_flag == 1))   /* tempA <= degC < tempB */
            {
                GPIO_LED1_OFF();
                GPIO_LED2_ON();
            }
            else if((cmp0_flag == 1) && (cmp1_flag == 0))   /* tempB <= degC */
            {
                GPIO_LED1_ON();
                GPIO_LED2_ON();
            }

            /* output flag clear */
            output_flag = 0;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  ADC conversion monitoring0 IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void INTADCCP0_IRQHandler(void)
{
    uint32_t data_shift;

    data_shift = data0 << 4;
    cmp0_flag = 1;
    if( data_shift <= CFG_ADC_CMP_35_BIG_THAN )
    {
        cmp1_flag = 0;
    }
    else
    {
        cmp1_flag = 1;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  ADC conversion monitoring1 IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void INTADCCP1_IRQHandler(void)
{
    uint32_t data_shift;

    data_shift = data0 << 4;
    cmp1_flag = 1;
    if( data_shift >= CFG_ADC_CMP_25_SMALL_THAN )
    {
        cmp0_flag = 0;
    }
    else
    {
        cmp0_flag = 1;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A00AC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   95: T32A ch1 TimmerA/C compare match detection / Over flow / under flow
  */
/*--------------------------------------------------*/
void INTT32A00AC_IRQHandler(void)
{
    if(Get_param_T32A_CH0_INTA1())
    {
        Clear_flag_T32A_CH0_INTA();
        if (adc_counter-- == 0)
        {
            fIntADC = 1;
            adc_counter = CFG_ADC_COUNT_TIME;
            ADC_C_SGL_Start();  /* ADC SGL start */
        }
        if (counter-- == 0)
        {
            output_flag = 1;
            counter = CFG_OUTPUT_COUNT_TIME;
        }
    }
}


/**
 *  @}
 */ /* End of group ADC_Monitor_Exported_functions */

/**
 *  @}
 */ /* End of group ADC_Monitor */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
