/** 
 *******************************************************************************
 * @file    main.c
 * @brief   ADC Software Trigger Sample Application. \n
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TSB_CMSIS_CORE.h"  /*!< Group Header file. */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "adc.h"
#include "cg.h"
#include "t32a.h"
#include "bsp.h"
#include "bsp_adc.h"
#include "bsp_led.h"
#include "uart_io.h"

/**
 *  @addtogroup Example Example
 *  @{
 */

/** 
 *  @addtogroup ADC_SoftwareTrigger ADC_SoftwareTrigger
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup ADC_SoftwareTrigger_config ADC_SoftwareTrigger Config
 *  @{
 */

/** 
 *  @addtogroup CountConfiguration Time Configuration
 *  @brief    Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_OUTPUT_COUNT_TIME           ((uint32_t)(5000))
#define CFG_ADC_COUNT_TIME              ((uint32_t)(1000))
/** 
 *  @}
 */ /* End of group CountConfiguration */

/** 
 *  @}
 */ /* End of group ADC_SoftwareTrigger_config */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup ADC_SoftwareTrigger_Private_variables ADC_SoftwareTrigger Private Variables
 *  @{
 */
static volatile uint32_t output_flag    = 0;        /*!< Output flag.   */
static volatile uint32_t fIntADC = 0;               /*!< ADC flag.      */
volatile uint32_t data0 = 0;                        /*!< ADC value      */
static uint32_t counter  = CFG_OUTPUT_COUNT_TIME;   /*!< ADC value output Time Counter  */
static uint32_t adc_counter = CFG_ADC_COUNT_TIME;   /* */
static int32_t degC;                                /*!< degC.          */
/**
 *  @}
 */ /* End of group ADC_SoftwareTrigger_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup ADC_SoftwareTrigger_Private_fuctions ADC_SoftwareTrigger Private Fuctions
 *  @{
 */

/**
 *  @}
 */ /* End of group ADC_SoftwareTrigger_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup ADC_SoftwareTrigger_Exported_functions ADC_SoftwareTrigger Exported Functions
 *  @{
 */
 
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{

    /*------------------------*/
    /* Port Setting           */
    /*------------------------*/
    /* Enable the peripheral clock of ADC unit C  */
    GPIO_ADC_C_enable_fsys();
    /* Configure ADC unit C  */
    GPIO_ADC_C_Config();

    /*------------------------*/
    /* UART Setting            */
    /*------------------------*/
    uart_io_initialize();

    /*------------------------*/
    /* ADC Setting            */
    /*------------------------*/
    /* Enable the peripheral clock of ADC Unit C */
    ADC_C_enable_fsys();
    /* ADC initial setting */
    ADC_C_Config();
    /* Convert Program Setting */
    Set_ADC_C_TSET0_value();

    /*------------------------*/
    /* T32A Interval TimerSetting */
    /*------------------------*/
    /* Enable the peripheral clock of T32A00 */
    T32A_CH0_enable_fsys();
    /* T32A Interval Timer Setting */
    T32A0_A_IntervalTimer_Config();
    /* Interrupt Enable */
    NVIC_EnableIRQ(INTT32A00AC_IRQn);   /* T32A ch.0 TimmerA compare match detection */
    /* Timer Start */
    T32A0_A_Start();

    /*------------------------*/
    /*          Loop          */
    /*------------------------*/ 
    while(1){
        if(fIntADC == 1)
        {
            fIntADC = 0;
            /* ADxREG0 [AIN0] AD Result Value Get */
            data0 = Get_ADC_C_REG0_CP_value();
        }

        if(output_flag == 1)
        {
            /* ADxREG0 [AIN0] AD Result Value Get */
            degC = bsp_get_thermistors_degC(data0);
            printf("Convert Result Value[THERMISTOR]:0x%04x (%d degrees)\n", data0, degC);
            printf("Convert Result Value[VR1]:0x%04x\n", data0);
            printf("Convert Result Value[VR2]:0x%04x\n\n", data0);

            /* output flag clear */
            output_flag = 0;

            NVIC_EnableIRQ(INTADCCNT_IRQn); /* ADC unit C Continuity program interrupt enable */
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  INTADCCNT IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   59: ADC unit C Continuity program interrupt
  */
/*--------------------------------------------------*/
void INTADCCNT_IRQHandler(void)
{
    /* ADxREG0 [AIN0] AD Result Value Get */
    data0 = Get_ADC_C_REG0_result_value();
}

/*--------------------------------------------------*/
/** 
  * @brief  INTT32A00AC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   95: T32A ch1 TimmerA/C compare match detection / Over flow / under flow
  */
/*--------------------------------------------------*/
void INTT32A00AC_IRQHandler(void)
{
    if(Get_param_T32A_CH0_INTA1())
    {
        Clear_flag_T32A_CH0_INTA();
        if (adc_counter-- == 0)
        {
            fIntADC = 1;
            adc_counter = CFG_ADC_COUNT_TIME;
            ADC_C_SGL_Start();  /* ADC SGL start */
        }
        if (counter-- == 0)
        {
            output_flag = 1;
            counter = CFG_OUTPUT_COUNT_TIME;
        }
    }
}


/**
 *  @}
 */ /* End of group ADC_SoftwareTrigger_Exported_functions */

/**
 *  @}
 */ /* End of group ADC_SoftwareTrigger */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
