/** 
 *******************************************************************************
 * @file    main.c
 * @brief   CG_EXINT Sample Application.\n
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TSB_CMSIS_CORE.h"  /*!< Group Header file. */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cg.h"
#include "dnf.h"
#include "except.h"
#include "siwdt.h"
#include "t32a.h"
#include "bsp.h"
#include "bsp_led.h"
#include "bsp_sw.h"


/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup CG_EXINT CG_EXINT Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup CG_EXINT_Private_macro CG_EXINT Private Macro
 *  @{
 */

/** 
 *  @addtogroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_LED_BLINK_CYCLE             ((uint32_t)(1000))
#define CFG_LED_BLINK_FRQ               ((uint32_t)(2))
#define CFG_LED_BLINK_DUTY              ((float)(0.5))
#define CFG_LED_ON_TIME   ((uint32_t)(CFG_LED_BLINK_CYCLE / CFG_LED_BLINK_FRQ * CFG_LED_BLINK_DUTY))
#define CFG_LED_OFF_TIME  ((uint32_t)(CFG_LED_BLINK_CYCLE / CFG_LED_BLINK_FRQ * (1 - CFG_LED_BLINK_DUTY)))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */

/** 
 *  @}
 */ /* End of group CG_EXINT_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup CG_EXINT_Private_define CG_EXINT Private Define
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    NORMAL_START,           /*!< Normal Mode    :START.     */
    NORMAL_MODE,            /*!< Normal Mode                */
    IDLE_START,             /*!< Idle Mode      :START.     */
    IDLE_MODE,              /*!< Idle Mode                  */
    STOP1_START,            /*!< Stop1 Mode     :START.     */
    STOP1_MODE,             /*!< Stop1 Mode                 */
    STOP1_END,              /*!< Stop1 Mode     :END        */
    PROC_ERROR              /*!< Error.                     */
} Processing;

/** 
 *  @}
 */ /* End of group CG_EXINT_Private_define */



/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup CG_EXINT_Private_variables CG_EXINT Private Variables
 *  @{
 */


static volatile Processing proc;                /*!< Processing mode.        */
static uint32_t counter =  CFG_LED_ON_TIME;     /*!< Time Counter            */
static bool     led1_on_off =  false;           /*!< Led blink state         */

/**
 *  @}
 */ /* End of group CG_EXINT_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup CG_EXINT_Private_fuctions CG_EXINT Private Fuctions
 *  @{
 */

/*--------------------------------------------------*/
/** 
  * @brief  Start Processing.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for Normal Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result normal_start(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    T32A_CH0_enable_fsys();
    T32A0_A_IntervalTimer_Config();
    NVIC_EnableIRQ( INTT32A00AC_IRQn );

    GPIO_LED1_ON();
    GPIO_LED2_OFF();
    GPIO_LED3_OFF();

    NVIC_DisableIRQ( INT21_IRQn );
    NVIC_EnableIRQ( INT17_IRQn );

    proc = NORMAL_MODE;

    T32A0_A_Start();

    return ( result );
}


/*--------------------------------------------------*/
/** 
  * @brief  Start Processing.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for IDLE Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result idle_start(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* WDT Disable          */
    /*----------------------*/
    disable_SIWDT();
    /*----------------------*/
    /* Flash Wait           */
    /*----------------------*/
    /* [FCSR0]<RDYBSY>   : Read(wait for done) */
    while( Get_param_FC_RDYBSY() != FCSR0_RDYBSY_R_DONE )
    {
        /* no processing */
    }
    Set_CG_STBY_value( CGSTBYCR_STBY_RW_IDLE );
    /*>>>Wait For Interrupt */
    GPIO_LED1_OFF();
    GPIO_LED2_ON();
    GPIO_LED3_OFF();
    /* Interrupt Enable */
    NVIC_EnableIRQ( INT21_IRQn );
    proc = IDLE_MODE;


    return ( result );
}

/*--------------------------------------------------*/
/** 
  * @brief  STOP1 Mode Start Processing.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   STOP1 Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result stop1_start(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* WDT Disable          */
    /*----------------------*/
    disable_SIWDT();
    /*----------------------*/
    /* Flash Wait           */
    /*----------------------*/
    /* [FCSR0]<RDYBSY>   : Read(wait for done) */
    while( Get_param_FC_RDYBSY() != FCSR0_RDYBSY_R_DONE )
    {
        /* no processing */
    }

    /*----------------------*/
    /* HOSC Warm-up Wait    */
    /*----------------------*/
    /* [CGWUPHCR]<WUEF>  : Read(wait for done) */
    Get_param_CG_WUEF();
    /*----------------------*/
    /* Internal HOSC Select */
    /*----------------------*/
    Set_param_CG_WUCLK();
    /*----------------------*/
    /* Internal HOSC Warm-up Time Set*/
    /*----------------------*/
    Set_CG_IOSC_WUPT_value();
    /*----------------------*/
    /* Set STBY             */
    /*----------------------*/
    /* [CGSTBYCR]<STBY>    :STOP1           */
    Set_CG_STBY_value( CGSTBYCR_STBY_RW_STOP1 );
    /*----------------------*/
    /* PLL Disable          */
    /*----------------------*/
    disable_PLL();
    /*----------------------*/
    /* Internal HOSC Enable */
    /*----------------------*/
    enable_IHOSC1();
    /*----------------------*/
    /* Warm-up Timer Start  */
    /*----------------------*/
    Set_param_CG_WUON();
    /*----------------------*/
    /* Check Warm-up Timer status */
    /*----------------------*/
    /* [CGWUPHCR]<WUEF>  : Read(wait for done) */
    Get_param_CG_WUEF();
    /*----------------------*/
    /* External HOSC Disable*/
    /*----------------------*/
    /* [CGOSCCR]<EOSCEN>   : EHOSC Disable  */
    disable_EHOSC();
    /*----------------------*/
    /* OFD Internal HOSC Disable*/
    /*----------------------*/
    /* [CGOSCCR]<IHOSC2EN> : OFD IHOSC Disable  */
    disable_IHOSC2();
    /*----------------------*/
    /* External HOSC Disable Check*/
    /*----------------------*/
    /* [CGOSCCR]<EOSCEN>   : Read  */
    while( Get_CG_EOSCEN_value() != CGOSCCR_EOSCEN_RW_UNUSE )
    {
        /* no processing */
    }
    /*----------------------*/
    /* OFD Internal HOSC Disable Wait*/
    /*----------------------*/
    /* [CGOSCCR]<IHOSC2F>  : Read(wait for OFD IHOSC Disable)  */
    while( Get_param_CG_IHOSC2F() != CGOSCCR_IHOSC2F_R_STOP )
    {
        /* no processing */
    }

    GPIO_LED1_OFF();
    GPIO_LED2_OFF();
    GPIO_LED3_ON();
    /*>>>Wait For Interrupt */

    NVIC_DisableIRQ( INTT32A00AC_IRQn );
    NVIC_DisableIRQ( INT17_IRQn );
    NVIC_EnableIRQ( INT21_IRQn );

    proc = STOP1_MODE;

    return (result);
}


/**
 *  @}
 */ /* End of group CG_EXINT_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup CG_EXINT_Exported_functions CG_EXINT Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    GPIO_LED_enable_fsys();
    GPIO_LED_Config();
    GPIO_SW_enable_fsys();
    GPIO_SW_Config();

    DNF_B_Config();
    DNF_C_Config( (uint32_t)DNF_NFCKCR_NFCKS_CLOCK_2 );

    INT21_enable_IMC069();
    INT17a_enable_IMC061();
    INT17b_enable_IMC062();

    proc = NORMAL_START;

    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while( proc != PROC_ERROR )
    {
        switch( proc )
        {
        /*--- Normal Mode     :START   ---*/
        case NORMAL_START:
            {
                if( normal_start() == TXZ_SUCCESS )
                {
                    proc = NORMAL_MODE;
                }
                else
                {
                    proc = PROC_ERROR;
                }
            }
            break;
        /*--- Normal Mode    :RUNNING ---*/
        case NORMAL_MODE:
            {
            }
            break;
        /*--- Idle Mode     :START   ---*/
        case IDLE_START:
            {
                if( idle_start() == TXZ_SUCCESS )
                {
                    proc = IDLE_MODE;
                    /*>>>Wait For Interrupt */
                    __WFI();
                }
                else
                {
                    proc = PROC_ERROR;
                }
            }
            break;
        /*--- Idle Mode    :RUNNING ---*/
        case IDLE_MODE:
            {
            }
            break;
        /*--- Stop1 Mode     :START   ---*/
        case STOP1_START:
            {
                if( stop1_start() == TXZ_SUCCESS )
                {
                    proc = STOP1_MODE;
                    /*>>>Wait For Interrupt */
                    __WFI();
                }
                else
                {
                    proc = PROC_ERROR;
                }
            }
            break;
        /*--- Idle Mode    :RUNNING ---*/
        case STOP1_MODE:
            {
            }
            break;
        /*--- Stop1 Mode :END ---*/
        case STOP1_END:
            {
                SystemInit();
                proc  = NORMAL_START;
            }
            break;
        /*--- Error                   ---*/
        case PROC_ERROR:
            /* no processing */
            break;
        /*--- default                 ---*/
        default:
            proc = PROC_ERROR;
            break;
        }
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  INT17 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void INT17_IRQHandler(void)
{
    bool intdet;
    if( proc == NORMAL_MODE )
    {
        intdet = Get_param_INT157FLG();
        if( intdet == true )
        {
            Clear_flag_INTNCLR_IMC061();
            proc = IDLE_START;
        }
        else
        {
            intdet = Get_param_INT158FLG();
            if( intdet == true )
            {
                Clear_flag_INTNCLR_IMC062();
                proc = STOP1_START;
            }
        }
        NVIC_DisableIRQ( INT17_IRQn );
        T32A0_A_Stop();
    }
}


/*--------------------------------------------------*/
/** 
  * @brief  INT21 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void INT21_IRQHandler(void)
{
    bool intdet;
    if(( proc == IDLE_MODE ) || ( proc == STOP1_MODE ))
    {
        intdet = Get_param_INT165FLG();
        if( intdet == true )
        {
            Clear_flag_INTPCLR_IMC069();
            NVIC_DisableIRQ(INT21_IRQn);
            if( proc == IDLE_MODE )
            {
                proc = NORMAL_START;
            }
            else
            {
                proc = STOP1_END;
            }
        }
    }
}


/*--------------------------------------------------*/
/** 
  * @brief  INTT32A00AC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   95: T32A ch1 TimmerA/C compare match detection / Over flow / under flow
  */
/*--------------------------------------------------*/
void INTT32A00AC_IRQHandler(void)
{
    uint32_t t32a_status;

    t32a_status = Get_param_T32A0_INTA1();
    if( t32a_status == T32AxSTA_INTA1_MASK )
    {
        Clear_flag_T32A0_INTA();
        if ( counter-- == 0 )
        {
            if( led1_on_off == false )
            {
                counter = CFG_LED_ON_TIME;
                led1_on_off = true;
                GPIO_LED1_ON();
            }
            else
            {
                counter = CFG_LED_OFF_TIME;
                led1_on_off = false;
                GPIO_LED1_OFF();
            }
        }
    }
}

/**
 *  @}
 */ /* End of group CG_EXINT_Exported_functions */

/**
 *  @}
 */ /* End of group CG_EXINT */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
