/** 
 *******************************************************************************
 * @file    main.c
 * @brief   EI2C Master Receive Sample Application.\n
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TSB_CMSIS_CORE.h"  /*!< Group Header file. */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cg.h"
#include "t32a.h"
#include "bsp.h"
#include "bsp_led.h"
#include "bsp_sw.h"
#include "bsp_ei2c.h"

#include "except.h"
#include "ei2c.h"
#include "dnf.h"

/**
 *  @addtogroup Example Example
 *  @{
 */

/** 
 *  @addtogroup EI2C_MasterReceive EI2C_MasterReceive
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_MasterReceive_config EI2C_MasterReceive Config
 *  @{
 */

#define DATA_LENGTH     (16)                /*!< Length of base data.          */
#define MCU_NAME        "TMPM4KNFYA"

/** 
 *  @}
 */ /* End of group EI2C_MasterReceive_config */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_MasterSlave_Private_define EI2C_MasterSlave Private Define
 *  @{
 */


/*----------------------------------*/
/** 
 * @enum   Processing
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                              */
    
    PROC_MASTER_START,      /*!< Master start                       */
    PROC_COMMAND_MSG,       /*!< Output first message.              */
    PROC_MASTER_STOP,       /*!<                                    */
    PROC_NACK_DETECT,       /*!<                                    */
    
    PROC_CMP,               /*!<                                    */
    
    PROC_RECEIVE_DATA,      /*!<                                    */
    
    PROC_ERROR              /*!< Error.                             */
} Processing;

/** 
 *  @}
 */ /* End of group EI2C_MasterSlave_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_MasterReceive_Private_variables EI2C_MasterReceive Private Variables
 *  @{
 */

static Processing proc;                       /*!< Processing mode.          */
static uint32_t   length;                     /*!< Length of Master Tx Data. */
uint8_t           rx_buf[DATA_LENGTH];        /*!< Rx Data of Master.        */

/**
 *  @}
 */ /* End of group EI2C_MasterReceive_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_MasterReceive_Private_fuctions EI2C_MasterReceive Private Fuctions
 *  @{
 */
void _PROC_MASTER_START(void);
void _PROC_COMMAND_MSG(void);
void _PROC_RECEIVE_DATA(void);
void _PROC_MASTER_STOP(void);
void _PROC_CMP(void);
void _PROC_NACK_DETECT(void);
/**
 *  @}
 */ /* End of group EI2C_MasterReceive_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_MasterReceive_Exported_functions EI2C_MasterReceive Exported Functions
 *  @{
 */
 
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*------------------------*/
    /* Port Setting           */
    /*------------------------*/
    /* Enable the peripheral clock of LED    */
    GPIO_LED_enable_fsys();
    /* Configure LED  */
    GPIO_LED_Config();
    
    /* Enable the peripheral clock of SW    */
    GPIO_SW_enable_fsys();
    /* Configure SW  */
    GPIO_SW_Config();
    
    /* Enable the peripheral clock of EI2C  */
    GPIO_EI2C_CH1_enable_fsys();
    /* Configure EI2C1_Master  */
    GPIO_EI2C1_Master_Config();
    
    /*------------------------*/
    /* Interrupt Setting      */
    /*------------------------*/
    /* DNF enable     */
    DNF_enable_fsys();
    /* DNF configuration */
    DNF_B_Config();   
    /* INT17a enable     */
    INT17a_enable_IMC061();
    /* Enable Interrupt  */
    NVIC_EnableIRQ( INT17_IRQn );
    
    /*------------------------*/
    /* EI2C Master Setting    */
    /*------------------------*/
    /* Clock suply for EI2C ch.1  */
    EI2C_CH1_enable_fsys();
    /* Configure EI2C1_Master     */
    EI2C1_Master_Config();
    
    /*------------------------*/
    /* LED Initial Setting    */
    /*------------------------*/
    GPIO_LED1_OFF();
    GPIO_LED3_OFF();
    
    /*------------------------*/
    /* Processing Initial mode*/
    /*------------------------*/
    proc = PROC_IDLE;
    
    /*------------------------*/
    /*          Loop          */
    /*------------------------*/ 
    while(1){
        switch(proc){
            case PROC_IDLE:                                     break;
            case PROC_MASTER_START:     _PROC_MASTER_START();   break;
            case PROC_COMMAND_MSG:      _PROC_COMMAND_MSG();    break;
            case PROC_RECEIVE_DATA:     _PROC_RECEIVE_DATA();   break;
            case PROC_MASTER_STOP:      _PROC_MASTER_STOP();    break;
            case PROC_CMP:              _PROC_CMP();            break;
            case PROC_NACK_DETECT:      _PROC_NACK_DETECT();    break;
            case PROC_ERROR:                                    break;
            default:                     proc = PROC_ERROR;     break;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_MASTER_START.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_MASTER_START(void)
{
    EI2C1_start_condition();
    
    while( Get_param_EI2C1_STCF() != FLAG_DETECT ){;}
    
    Clear_flag_EI2C1_STCF();
    
    length = 0;
    
    proc = PROC_COMMAND_MSG;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_COMMAND_MSG.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_COMMAND_MSG(void)
{

    Set_EI2C1_DBRT_value( (( SLAVE_ADDRESS << 1) | 1U) );
    
    proc = PROC_RECEIVE_DATA;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_RECEIVE_DATA.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_RECEIVE_DATA(void)
{
    if( Get_param_EI2C1_NACK() != FLAG_DETECT )
    {
        do
        {
            while( Get_param_EI2C1_RBF() != FLAG_DETECT ){;}
            rx_buf[length] = Get_EI2C1_DBRR_value();
            length++;

        }while( length < (DATA_LENGTH - 1));

        Set_param_EI2C1_ACKWAIT();

        while( Get_param_EI2C1_RBF() != FLAG_DETECT ){;}

        rx_buf[length] = Get_EI2C1_DBRR_value();
        proc = PROC_MASTER_STOP;
    }
    else
    {
        proc = PROC_NACK_DETECT;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_MASTER_STOP.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_MASTER_STOP(void)
{

    Set_param_EI2C1_ACKSEL();

    EI2C1_stop_condition();

    while( Get_param_EI2C1_SPCF() != FLAG_DETECT ){;}

    Clear_allflag_EI2C1_ASR1();
    Clear_EI2C1_ACR1();

    NVIC_EnableIRQ( INT17_IRQn );
    
    GPIO_LED1_FLIP();
    GPIO_LED3_OFF();

    proc = PROC_CMP;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_CMP.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_CMP(void)
{
    if( strcmp((char *)rx_buf , MCU_NAME ) != 0)
    {
        GPIO_LED3_ON();
    }

    proc = PROC_IDLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_NACK_DETECT.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_NACK_DETECT(void)
{
    EI2C1_stop_condition();
    Clear_flag_EI2C1_NACK();

    while( Get_param_EI2C1_SPCF() != FLAG_DETECT ){;}

    Clear_allflag_EI2C1_ASR1();
    NVIC_EnableIRQ( INT17_IRQn );
    GPIO_LED3_ON();
    proc = PROC_IDLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  INT17 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   17:  Interrupt pin 17a/17b
  */
/*--------------------------------------------------*/
void INT17_IRQHandler(void)
{
    Clear_flag_INTNCLR_IMC061();

    NVIC_DisableIRQ( INT17_IRQn );

    GPIO_LED3_OFF();
    proc = PROC_MASTER_START;
}

/**
 *  @}
 */ /* End of group EI2C_MasterReceive_Exported_functions */

/**
 *  @}
 */ /* End of group EI2C_MasterReceive */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
