/** 
 *******************************************************************************
 * @file    main.c
 * @brief   EI2C Master Trans Sample Application.\n
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TSB_CMSIS_CORE.h"  /*!< Group Header file. */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cg.h"
#include "t32a.h"
#include "bsp.h"
#include "bsp_led.h"
#include "bsp_sw.h"
#include "bsp_ei2c.h"

#include "except.h"
#include "ei2c.h"
#include "dnf.h"

/**
 *  @addtogroup Example Example
 *  @{
 */

/** 
 *  @addtogroup EI2C_MasterTrans EI2C_MasterTrans
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_MasterTrans_config EI2C_MasterTrans Config
 *  @{
 */

#define MCU_NAME        "TMPM4KNFYA"

/** 
 *  @}
 */ /* End of group EI2C_MasterTrans_config */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_MasterSlave_Private_define EI2C_MasterSlave Private Define
 *  @{
 */


/*----------------------------------*/
/** 
 * @enum   Processing
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                              */
    
    PROC_MASTER_START,      /*!< Master start                       */
    PROC_MASTER_STOP,       /*!<                                    */
    
    PROC_TRANS_DATA,        /*!<                                    */
    
    PROC_COMMAND_MSG,       /*!< Output first message.              */
    PROC_NACK_DETECT,       /*!<                                    */
    
    PROC_MASTER_ERROR,      /*!< Master Error.                      */
    PROC_ERROR              /*!< Error.                             */
} Processing;

/** 
 *  @}
 */ /* End of group EI2C_MasterSlave_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_MasterTrans_Private_variables EI2C_MasterTrans Private Variables
 *  @{
 */
#define DATA_LENGTH     (16)                    /*!< Length of base data.       */

static Processing proc;                         /*!< Processing mode.           */
static uint32_t   length;                       /*!< Length of Master Tx Data.  */
static uint8_t    tx_buf[DATA_LENGTH];          /*!< Tx Data of Master.         */

/**
 *  @}
 */ /* End of group EI2C_MasterTrans_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_MasterTrans_Private_fuctions EI2C_MasterTrans Private Fuctions
 *  @{
 */
void _PROC_MASTER_START(void);
void _PROC_COMMAND_MSG(void);
void _PROC_TRANS_DATA(void);
void _PROC_MASTER_STOP(void);
void _PROC_NACK_DETECT(void);
void _PROC_MASTER_ERROR(void);
/**
 *  @}
 */ /* End of group EI2C_MasterTrans_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_MasterTrans_Exported_functions EI2C_MasterTrans Exported Functions
 *  @{
 */
 
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*------------------------*/
    /* Port Setting           */
    /*------------------------*/
    /* Enable the peripheral clock of LED    */
    GPIO_LED_enable_fsys();
    /* Configure LED  */
    GPIO_LED_Config();
    
    /* Enable the peripheral clock of SW     */
    GPIO_SW_enable_fsys();
    /* Configure SW  */
    GPIO_SW_Config();
    
    /* Enable the peripheral clock of EI2C   */
    GPIO_EI2C_CH1_enable_fsys();
    /* Configure EI2C1_Master  */
    GPIO_EI2C1_Master_Config();
    
    /*------------------------*/
    /* Interrupt Setting      */
    /*------------------------*/
    /* DNF enable     */
    DNF_enable_fsys();
    /* DNF configuration */
    DNF_B_Config();
    /* INT17a enable     */
    INT17a_enable_IMC061();
    /* Enable Interrupt */
    NVIC_EnableIRQ( INT17_IRQn );
    
    /*------------------------*/
    /* EI2C Master Setting    */
    /*------------------------*/
    /* Clock suply for EI2C ch.1  */
    EI2C_CH1_enable_fsys();
    /* Configure EI2C1_Master     */
    EI2C1_Master_Config();
    
    /*------------------------*/
    /* LED Initial Setting    */
    /*------------------------*/
    GPIO_LED1_OFF();
    GPIO_LED3_OFF();
    
    /*------------------------*/
    /* Processing Initial mode*/
    /*------------------------*/
    proc = PROC_IDLE;
    
    /*------------------------*/
    /* Send Data Setting      */
    /*------------------------*/
    strcpy( (char *)tx_buf , MCU_NAME );
    
    /*------------------------*/
    /*          Loop          */
    /*------------------------*/
    while(1){
        switch(proc){
            case PROC_IDLE:                                     break;
            case PROC_MASTER_START:     _PROC_MASTER_START();   break;
            case PROC_COMMAND_MSG:      _PROC_COMMAND_MSG();    break;
            case PROC_TRANS_DATA:       _PROC_TRANS_DATA();     break;
            case PROC_MASTER_STOP:      _PROC_MASTER_STOP();    break;
            case PROC_NACK_DETECT:      _PROC_NACK_DETECT();    break;
            case PROC_MASTER_ERROR:     _PROC_MASTER_ERROR();   break;
            case PROC_ERROR:                                    break;
            default:                     proc = PROC_ERROR;     break;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_MASTER_START.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_MASTER_START(void)
{
    EI2C1_start_condition();

    while( Get_param_EI2C1_STCF() != FLAG_DETECT){;}

    Clear_flag_EI2C1_STCF();
    length = 0;
    proc = PROC_COMMAND_MSG;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_COMMAND_MSG.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_COMMAND_MSG(void)
{
    Set_EI2C1_DBRT_value( ( SLAVE_ADDRESS << 1) );

    proc = PROC_TRANS_DATA;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_TRANS_DATA.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_TRANS_DATA(void)
{
    do
    {
        if( Get_param_EI2C1_NACK() != FLAG_NOT_DETECT )
        {
            proc = PROC_NACK_DETECT;
            return;
        }
        /* Not Detect = 0  */
    } while ( Get_param_EI2C1_TBE() != FLAG_DETECT );

    Set_EI2C1_DBRT_value( tx_buf[length] );
    length++;

    if( Get_param_EI2C1_TEND() == FLAG_DETECT )
    {
        Clear_flag_EI2C1_TEND();
    }

    if( length == DATA_LENGTH )
    {
        while( Get_param_EI2C1_TEND() != FLAG_DETECT ){;}

        proc = PROC_MASTER_STOP;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_MASTER_STOP.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_MASTER_STOP(void)
{

    EI2C1_stop_condition();
    Clear_flag_EI2C1_TEND();

    do
    {
        if( Get_param_EI2C1_ESP() == FLAG_DETECT)
        {
            proc = PROC_MASTER_ERROR;
            return;
        }
    } while( Get_param_EI2C1_SPCF() != FLAG_DETECT );

    Clear_allflag_EI2C1_ASR1();

    NVIC_EnableIRQ( INT17_IRQn );

    GPIO_LED1_FLIP();
    proc = PROC_IDLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_NACK_DETECT.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_NACK_DETECT(void)
{
    EI2C1_stop_condition();
    Clear_flag_EI2C1_NACK();

    while( Get_param_EI2C1_SPCF() != FLAG_DETECT ){;}

    Clear_allflag_EI2C1_ASR1();

    NVIC_EnableIRQ( INT17_IRQn );

    GPIO_LED3_ON();
    proc = PROC_IDLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_MASTER_ERROR.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_MASTER_ERROR(void)
{
    Clear_allflag_EI2C1_ASR1();

    NVIC_EnableIRQ( INT17_IRQn );

    GPIO_LED1_OFF();
    GPIO_LED3_ON();

    proc = PROC_IDLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  INT17 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   17:  Interrupt pin 17a/17b
  */
/*--------------------------------------------------*/
void INT17_IRQHandler(void)
{
    Clear_flag_INTNCLR_IMC061();

    NVIC_DisableIRQ( INT17_IRQn );

    GPIO_LED3_OFF();
    proc = PROC_MASTER_START;
}

/**
 *  @}
 */ /* End of group EI2C_MasterTrans_Exported_functions */

/**
 *  @}
 */ /* End of group EI2C_MasterTrans */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
