/** 
 *******************************************************************************
 * @file    main.c
 * @brief   EI2C Slave Receive Sample Application.\n
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TSB_CMSIS_CORE.h"  /*!< Group Header file. */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cg.h"
#include "bsp.h"
#include "bsp_led.h"
#include "bsp_sw.h"
#include "bsp_ei2c.h"

#include "except.h"
#include "ei2c.h"
#include "dnf.h"

/**
 *  @addtogroup Example Example
 *  @{
 */

/** 
 *  @addtogroup EI2C_SlaveReceive EI2C_SlaveReceive
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_SlaveReceive_config EI2C_SlaveReceive Config
 *  @{
 */

#define MCU_NAME        "TMPM4KNFYA"
#define DATA_LENGTH     (16)                /*!< Length of base data.          */

/** 
 *  @}
 */ /* End of group EI2C_SlaveReceive_config */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_MasterSlave_Private_define EI2C_MasterSlave Private Define
 *  @{
 */


/*----------------------------------*/
/** 
 * @enum   Processing
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                              */
    
    PROC_SLAVE_START,       /*!<                                    */
    PROC_SLAVE_STOP,        /*!<                                    */
    PROC_CMP,               /*!<                                    */
    
    PROC_SLAVE_ERROR,       /*!< Slave Error.                       */
    PROC_ERROR              /*!< Error.                             */
} Processing;

/** 
 *  @}
 */ /* End of group EI2C_MasterSlave_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_SlaveReceive_Private_variables EI2C_SlaveReceive Private Variables
 *  @{
 */

static Processing proc;                         /*!< Processing mode.           */
static uint32_t   length;                       /*!< Length of Master Tx Data.  */
static uint8_t    rx_buf[DATA_LENGTH];          /*!< Rx Data of Master.         */

/**
 *  @}
 */ /* End of group EI2C_SlaveReceive_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_SlaveReceive_Private_fuctions EI2C_SlaveReceive Private Fuctions
 *  @{
 */

void _PROC_SLAVE_START(void);
void _PROC_SLAVE_STOP(void);
void _PROC_SLAVE_ERROR(void);
void _PROC_CMP(void);

/**
 *  @}
 */ /* End of group EI2C_SlaveReceive_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_SlaveReceive_Exported_functions EI2C_SlaveReceive Exported Functions
 *  @{
 */
 
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*------------------------*/
    /* Port Setting           */
    /*------------------------*/
    /* Enable the peripheral clock of LED    */
    GPIO_LED_enable_fsys();
    /* Configure LED  */
    GPIO_LED_Config();
    
    /* Enable the peripheral clock of SW    */
    GPIO_SW_enable_fsys();
    /* Configure SW  */
    GPIO_SW_Config();
    
    /* Enable the peripheral clock of EI2C  */
    GPIO_EI2C_CH1_enable_fsys();
    /* Configure EI2C1_Slave  */
    GPIO_EI2C1_Slave_Config();
    
    /*------------------------*/
    /* Interrupt Setting      */
    /*------------------------*/
    /* DNF enable     */
    DNF_enable_fsys();
    /* DNF configuration */
    DNF_B_Config();
    /* INT17a enable     */
    INT17a_enable_IMC061();
    /* Enable Interrupt */
    NVIC_EnableIRQ(INT17_IRQn);
    
    /*------------------------*/
    /* EI2C Master Setting    */
    /*------------------------*/
    /* Clock suply for EI2C ch.1  */
    EI2C_CH1_enable_fsys();
    /* Configure EI2C1_Slave      */
    EI2C1_Slave_Config();
    
    /*------------------------*/
    /* LED Initial Setting    */
    /*------------------------*/
    GPIO_LED2_OFF();
    GPIO_LED3_OFF();
    
    /*------------------------*/
    /* Processing Initial mode*/
    /*------------------------*/
    proc = PROC_IDLE;
    
    /*------------------------*/
    /*          Loop          */
    /*------------------------*/
    while(1){
        switch(proc){
            case PROC_IDLE:                                     break;
            case PROC_SLAVE_START:      _PROC_SLAVE_START();    break;
            case PROC_SLAVE_STOP:       _PROC_SLAVE_STOP();     break;
            case PROC_CMP:              _PROC_CMP();            break;
            case PROC_SLAVE_ERROR:      _PROC_SLAVE_ERROR();    break;
            case PROC_ERROR:                                    break;
            default:                     proc = PROC_ERROR;     break;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_SLAVE_START.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_SLAVE_START(void)
{
    length = 0;
    do
    {
        if( Get_param_EI2C1_RBF() == FLAG_DETECT )
        {
            if (length < DATA_LENGTH )
            {
                rx_buf[ length ] = Get_EI2C1_DBRR_value();
                length++;
            }
            else
            {
                Clear_flag_EI2C1_RBF();
            }
        }
        
        if( Get_param_EI2C1_ESP() == FLAG_DETECT )
        {
            proc = PROC_SLAVE_ERROR;
            return;
        }
        
    } while( Get_param_EI2C1_SPCF() != FLAG_DETECT );
    
    proc = PROC_SLAVE_STOP;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_SLAVE_STOP.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_SLAVE_STOP(void)
{
    Clear_allflag_EI2C1_ASR1();

    NVIC_EnableIRQ( INT17_IRQn );

    GPIO_LED2_FLIP();
    GPIO_LED3_OFF();
    proc = PROC_CMP;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_SLAVE_ERROR.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_SLAVE_ERROR(void)
{
    Clear_allflag_EI2C1_ASR1();

    NVIC_EnableIRQ( INT17_IRQn );

    GPIO_LED2_OFF();
    GPIO_LED3_ON();
    Disable_EI2C1_AAR1_SA1E();
    proc = PROC_IDLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_CMP.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_CMP(void)
{
    if( strcmp((char *)rx_buf , MCU_NAME ) != 0 )
    {
        GPIO_LED3_ON();
    }

    Disable_EI2C1_AAR1_SA1E();
    proc = PROC_IDLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  INT17 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   17:  Interrupt pin 17a/17b
  */
/*--------------------------------------------------*/
void INT17_IRQHandler(void)
{
    Clear_flag_INTNCLR_IMC061();

    NVIC_DisableIRQ( INT17_IRQn );

    GPIO_LED3_OFF();
    Enable_EI2C1_AAR1_SA1E();
    proc = PROC_SLAVE_START;
}

/**
 *  @}
 */ /* End of group EI2C_SlaveReceive_Exported_functions */

/**
 *  @}
 */ /* End of group EI2C_SlaveReceive */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
