/** 
 *******************************************************************************
 * @file    main.c
 * @brief   EI2C Slave Trans Sample Application.\n
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TSB_CMSIS_CORE.h"  /*!< Group Header file. */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cg.h"
#include "t32a.h"
#include "bsp.h"
#include "bsp_led.h"
#include "bsp_sw.h"
#include "bsp_ei2c.h"

#include "except.h"
#include "ei2c.h"
#include "dnf.h"
     
/**
 *  @addtogroup Example Example
 *  @{
 */

/** 
 *  @addtogroup EI2C_SlaveTrans EI2C_SlaveTrans
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_SlaveTrans_config EI2C_SlaveTrans Config
 *  @{
 */

#define MCU_NAME        "TMPM4KNFYA"
#define SINGLE_CHAR      ' '

/** 
 *  @}
 */ /* End of group EI2C_SlaveTrans_config */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_MasterSlave_Private_define EI2C_MasterSlave Private Define
 *  @{
 */


/*----------------------------------*/
/** 
 * @enum   Processing
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                              */
    
    PROC_SLAVE_STOP,        /*!<                                    */
    PROC_SLAVE_COMMAND,     /*!<                                    */
    PROC_SLAVE_ERROR,       /*!<                                    */
    
    PROC_TRANS_DATA,        /*!<                                    */
    
    PROC_NACK_DETECT,       /*!<                                    */
    
    PROC_ERROR              /*!< Error.                             */
} Processing;

/** 
 *  @}
 */ /* End of group EI2C_MasterSlave_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_SlaveTrans_Private_variables EI2C_SlaveTrans Private Variables
 *  @{
 */
#define DATA_LENGTH     (16)                    /*!< Length of base data.       */

static Processing proc;                         /*!< Processing mode.           */
static uint32_t   length;                       /*!< Length of Master Tx Data.  */
static uint8_t    tx_buf[DATA_LENGTH];          /*!< Tx Data of Master.         */

/**
 *  @}
 */ /* End of group EI2C_SlaveTrans_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_SlaveTrans_Private_fuctions EI2C_SlaveTrans Private Fuctions
 *  @{
 */

void _PROC_SLAVE_COMMAND(void);
void _PROC_TRANS_DATA(void);
void _PROC_SLAVE_STOP(void);
void _PROC_NACK_DETECT(void);
void _PROC_SLAVE_ERROR(void);

/**
 *  @}
 */ /* End of group EI2C_SlaveTrans_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_SlaveTrans_Exported_functions EI2C_SlaveTrans Exported Functions
 *  @{
 */
 
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*------------------------*/
    /* Port Setting           */
    /*------------------------*/
    /* Enable the peripheral clock of LED    */
    GPIO_LED_enable_fsys();
    /* Configure LED  */
    GPIO_LED_Config();
    
    /* Enable the peripheral clock of SW    */
    GPIO_SW_enable_fsys();
    /* Configure SW  */
    GPIO_SW_Config();
    
    /* Enable the peripheral clock of EI2C  */
    GPIO_EI2C_CH1_enable_fsys();
    /* Configure EI2C1_Slave  */
    GPIO_EI2C1_Slave_Config();
    
    /*------------------------*/
    /* Interrupt Setting      */
    /*------------------------*/
    /* DNF enable     */
    DNF_enable_fsys();
    /* DNF configuration */
    DNF_B_Config();
    /* INT17a enable     */
    INT17a_enable_IMC061();
    /* Enable Interrupt */
    NVIC_EnableIRQ(INT17_IRQn);
    
    /*------------------------*/
    /* EI2C Master Setting    */
    /*------------------------*/
    /* Clock suply for EI2C ch.1  */
    EI2C_CH1_enable_fsys();
    /* Configure EI2C1_Slave      */
    EI2C1_Slave_Config();
    
    /*------------------------*/
    /* LED Initial Setting    */
    /*------------------------*/
    GPIO_LED2_OFF();
    GPIO_LED3_OFF();
    
    /*------------------------*/
    /* Processing Initial mode*/
    /*------------------------*/
    proc = PROC_IDLE;

    /*------------------------*/
    /* Send Data Setting      */
    /*------------------------*/
    strcpy( (char *)tx_buf , MCU_NAME );
    
    /*------------------------*/
    /*          Loop          */
    /*------------------------*/
    while(1){
        switch(proc){
            case PROC_IDLE:                                     break;
            case PROC_SLAVE_COMMAND:    _PROC_SLAVE_COMMAND();  break;
            case PROC_TRANS_DATA:       _PROC_TRANS_DATA();     break;
            case PROC_SLAVE_STOP:       _PROC_SLAVE_STOP();     break;
            case PROC_NACK_DETECT:      _PROC_NACK_DETECT();    break;
            case PROC_SLAVE_ERROR:      _PROC_SLAVE_ERROR();    break;
            case PROC_ERROR:                                    break;
            default:                     proc = PROC_ERROR;     break;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_SLAVE_COMMAND.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_SLAVE_COMMAND(void)
{

    while( Get_param_EI2C1_AAS1() != FLAG_DETECT ){;}

    Clear_flag_EI2C1_AAS1();

    while( Get_param_EI2C1_BC() != 0x00 ){;}

    do 
    {
        if ( Get_param_EI2C1_NACK( ) == FLAG_DETECT )
        {
            proc = PROC_NACK_DETECT;
        }
    } while( Get_param_EI2C1_TBE() != FLAG_DETECT );


    length = 0;
    proc = PROC_TRANS_DATA;

}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_TRANS_DATA.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_TRANS_DATA(void)
{
    do
    {
        if( length < (sizeof(MCU_NAME)) )
        {
            Set_EI2C1_DBRT_value( tx_buf[length] );
            length++;
        }
        else
        {
            Set_EI2C1_DBRT_value( SINGLE_CHAR );
        }
        Clear_flag_EI2C1_TEND();

        do {
            if( Get_param_EI2C1_ESP() == FLAG_DETECT )
            {
                proc = PROC_SLAVE_ERROR;
                return;
            }
        } while( Get_param_EI2C1_TEND() != FLAG_DETECT );
    } while( Get_param_EI2C1_NACK() != FLAG_DETECT );

    Clear_flag_EI2C1_NACK();
    Clear_flag_EI2C1_TEND();

    do
    {
        if( Get_param_EI2C1_ESP() == FLAG_DETECT )
        {
            proc = PROC_SLAVE_ERROR;
            return;
        }
    } while( Get_param_EI2C1_SPCF() != FLAG_DETECT);
    proc = PROC_SLAVE_STOP;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_SLAVE_STOP.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_SLAVE_STOP(void)
{

    Clear_allflag_EI2C1_ASR1();

    NVIC_EnableIRQ( INT17_IRQn );

    GPIO_LED2_FLIP();
    GPIO_LED3_OFF();
    Disable_EI2C1_AAR1_SA1E();
    proc = PROC_IDLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_NACK_DETECT.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_NACK_DETECT(void)
{
    Clear_flag_EI2C1_NACK();

    while( Get_param_EI2C1_SPCF() != FLAG_DETECT ){;}

    Clear_allflag_EI2C1_ASR1();

    NVIC_EnableIRQ( INT17_IRQn );

    GPIO_LED2_OFF();
    GPIO_LED3_ON();
    Disable_EI2C1_AAR1_SA1E();
    proc = PROC_IDLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_SLAVE_ERROR.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_SLAVE_ERROR(void)
{
    Clear_allflag_EI2C1_ASR1();

    NVIC_EnableIRQ( INT17_IRQn );

    GPIO_LED2_OFF();
    GPIO_LED3_ON();
    Disable_EI2C1_AAR1_SA1E();
    proc = PROC_IDLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  INT17 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   17:  Interrupt pin 17a/17b
  */
/*--------------------------------------------------*/
void INT17_IRQHandler(void)
{
    Clear_flag_INTNCLR_IMC061();

    NVIC_DisableIRQ( INT17_IRQn );

    GPIO_LED3_OFF();
    Enable_EI2C1_AAR1_SA1E();
    proc = PROC_SLAVE_COMMAND;
}

/**
 *  @}
 */ /* End of group EI2C_SlaveTrans_Exported_functions */

/**
 *  @}
 */ /* End of group EI2C_SlaveTrans */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
