/** 
 *******************************************************************************
 * @file    main.c
 * @brief   Flash Code Sample Application.\n
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TSB_CMSIS_CORE.h"  /*!< Group Header file. */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cg.h"
#include "t32a.h"
#include "bsp.h"
#include "bsp_led.h"
#include "bsp_sw.h"

#include "except.h"
#include "flash.h"
#include "dnf.h"

/**
 *  @addtogroup Example Example
 *  @{
 */

/** 
 *  @addtogroup Flash_Code Flash_Code
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup Flash_Code_config Flash_Code Config
 *  @{
 */
/* DEMO ROM SIZE */
#define DEMO_SIZE               FC_PAGE_SIZE       /*   0x1000 */
/*#define DEMO_SIZE             FC_BLOCK_SIZE   */ /*   0x8000 */
/*#define DEMO_SIZE             FC_PAGE_SIZE    */ /* 0x320000 */

#define DEMO_AREA_A             FC_CODE_FLASH_PAGE8
#define DEMO_AREA_B             FC_CODE_FLASH_PAGE16
/*--------------------------------------------------*/
/** 
  * @brief  data a
  */
/*--------------------------------------------------*/
#if defined ( __CC_ARM   )      /* RealView Compiler */
const char data_a[] __attribute__((section("DATA_A"))) =
#elif defined ( __ICCARM__ )    /* IAR Compiler */
const char data_a[] __attribute__((section("STR_A"))) =
#endif
{
    "Execute Program A \n"
};

/** 
 *  @addtogroup FLASH_Code_Exported_functions FLASH_Code Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  data b
  */
/*--------------------------------------------------*/
#if defined ( __CC_ARM   )      /* RealView Compiler */
const char data_b[] __attribute__((section("DATA_B"))) =
#elif defined ( __ICCARM__ )    /* IAR Compiler */
const char data_b[] __attribute__((section("STR_B"))) =
#endif
{
    "Execute Program B \n"
};

/** 
 *  @}
 */ /* End of group Flash_Code_config */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup Flash_Code_Private_define Flash_Code Private Define
 *  @{
 */


/*----------------------------------*/
/** 
 * @enum   Processing
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                              */
    
    PROC_START,             /*!< Start.                             */
    PROC_RAMTRANSFER,       /*!< RamTransfer.                       */
    PROC_ERASING,           /*!< Erasing.                           */
    PROC_REWRITING,         /*!< ReWriting.                         */
    
    PROC_AREA_ERASING,      /*!< Area Erasing.                      */
    PROC_PAGE_WRITING,      /*!< Page Writing.                      */
    PROC_PAGE_ERASING,      /*!< Page Erasing.                      */
    PROC_BLOCK_ERASING,     /*!< Block Erasing.                     */
    
    PROC_ERASE_ERROR,       /*!< Erase_Error.                       */
    PROC_WRITE_ERROR,       /*!< Write Error.                       */
    
    PROC_ERROR              /*!< Error.                             */
} Processing;

/** 
 *  @}
 */ /* End of group Flash_Code_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup Flash_Code_Private_variables Flash_Code Private Variables
 *  @{
 */
static Processing proc;                         /*!< Processing mode.          */

/**
 *  @}
 */ /* End of group Flash_Code_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup Flash_Code_Private_fuctions Flash_Code Private Fuctions
 *  @{
 */
void _PROC_IDLE(void);
void _PROC_RAMTRANSFER(void);
void _PROC_ERASING(void);
void _PROC_REWRITING(void);
void _PROC_ERASE_ERROR(void);
void _PROC_WRITE_ERROR(void);

/**
 *  @}
 */ /* End of group Flash_Code_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup Flash_Code_Exported_functions Flash_Code Exported Functions
 *  @{
 */
 
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*------------------------*/
    /* Port Setting           */
    /*------------------------*/
    /* Enable the peripheral clock of LED    */
    GPIO_LED_enable_fsys();
    /* Configure LED  */
    GPIO_LED_Config();
    
    /* Enable the peripheral clock of SW     */
    GPIO_SW_enable_fsys();
    /* Configure SW  */
    GPIO_SW_Config();
    
    /* enable INT17a */
    INT17a_enable_IMC061();
    /* enable IHOSC1 */
    enable_IHOSC1();
    /* Internal HOSC Warm-up Time Set*/
    Set_CG_IOSC_WUPT_value();
    /* Warm-up Timer Start  */
    Set_param_CG_WUON();
    /* Check Warm-up Timer status */
    /* [CGWUPHCR]<WUEF>  : Read(wait for done) */
    Get_param_CG_WUEF();

    /* Enable the peripheral clock of DNF_B  */
    DNF_enable_fsys();
    /* Configure DNF_B  */
    DNF_B_Config();

    /*------------------------*/
    /* Processing Initial mode*/
    /*------------------------*/
    proc = PROC_IDLE;
    
    /*------------------------*/
    /*          Loop          */
    /*------------------------*/
    while(1){
        switch(proc){
            case PROC_IDLE:             _PROC_IDLE();           break;
            case PROC_START:                                    break;
            case PROC_RAMTRANSFER:      _PROC_RAMTRANSFER();    break;
            case PROC_ERASING:          _PROC_ERASING();        break;
            case PROC_REWRITING:        _PROC_REWRITING();      break;
            case PROC_ERASE_ERROR:      _PROC_ERASE_ERROR();    break;
            case PROC_WRITE_ERROR:      _PROC_WRITE_ERROR();    break;
            case PROC_ERROR:                                    break;
            default:                     proc = PROC_ERROR;     break;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_IDLE.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_IDLE(void)
{
    char cmp_data[20] = "Execute Program A \n";

    if( strncmp(cmp_data , data_a , sizeof(cmp_data)) == 0 )
    {
        GPIO_LED1_ON();
        GPIO_LED2_OFF();
        GPIO_LED3_ON();
        GPIO_LED4_OFF();
    }
    else if( strncmp(cmp_data , data_b , sizeof(cmp_data)) == 0 )
    {
        GPIO_LED1_OFF();
        GPIO_LED2_ON();
        GPIO_LED3_OFF();
        GPIO_LED4_ON();
    }

    NVIC_EnableIRQ( INT17_IRQn );

    proc = PROC_START;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_RAMTRANSFER.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_RAMTRANSFER(void)
{
    GPIO_LED1_OFF();
    GPIO_LED2_OFF();
    GPIO_LED3_OFF();
    GPIO_LED4_OFF();
    
    copy_fc_func();
    copy_user_program( DEMO_SIZE );
    proc = PROC_ERASING;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_ERASING.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_ERASING(void)
{
    if( fc_erase_page_code_flash( DEMO_AREA_A , 1 ) == TXZ_SUCCESS )
    {
        if( fc_erase_page_code_flash( DEMO_AREA_B , 1 ) == TXZ_SUCCESS )
        {
            proc = PROC_REWRITING;
        }
        else
        {
            proc = PROC_ERASE_ERROR;
        }
    }
    else
    {
        proc = PROC_ERASE_ERROR;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_REWRITING.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_REWRITING(void)
{
    if( rewrite_user_program(DEMO_SIZE) == TXZ_SUCCESS )
    {
        proc = PROC_IDLE;
    }
    else
    {
        proc = PROC_WRITE_ERROR;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_ERASE_ERROR.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_ERASE_ERROR(void)
{
    GPIO_LED1_OFF();
    GPIO_LED2_OFF();
    GPIO_LED3_ON();
    GPIO_LED4_OFF();
    
    proc = PROC_ERROR;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_WRITE_ERROR.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_WRITE_ERROR(void)
{
    GPIO_LED1_OFF();
    GPIO_LED2_OFF();
    GPIO_LED3_OFF();
    GPIO_LED4_ON();
    
    proc = PROC_ERROR;
}

/*--------------------------------------------------*/
/** 
  * @brief  INT17 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   17:  Interrupt pin 17a/17b
  */
/*--------------------------------------------------*/
void INT17_IRQHandler(void)
{
    if( Get_param_INT157FLG() )
    {
        Clear_flag_INTNCLR_IMC061();

        NVIC_DisableIRQ( INT17_IRQn );

        proc = PROC_RAMTRANSFER;
    }
}

/**
 *  @}
 */ /* End of group Flash_Code_Exported_functions */

/**
 *  @}
 */ /* End of group Flash_Code */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
