/** 
 *******************************************************************************
 * @file    main.c
 * @brief   Flash Data Sample Application.\n
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TSB_CMSIS_CORE.h"  /*!< Group Header file. */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cg.h"
#include "t32a.h"
#include "bsp.h"
#include "bsp_led.h"
#include "bsp_sw.h"

#include "except.h"
#include "flash.h"
#include "dnf.h"

/**
 *  @addtogroup Example Example
 *  @{
 */

/** 
 *  @addtogroup Flash_Data Flash_Data
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup Flash_Data_config Flash_Data Config
 *  @{
 */
#define CMF_DATA_AREA_ALL           FC_DATA_FLASH_AREA4
#define CMF_DATA_AREA_A             FC_DATA_FLASH_PAGE0
#define CMF_DATA_AREA_B             FC_DATA_FLASH_PAGE1
#define CMF_DATA_AREA_C             FC_DATA_FLASH_BLOCK0

#define CMF_DATA_SIZE_A             FC_DATA_PAGE_SIZE       /*   0x100 */

/** 
 *  @}
 */ /* End of group Flash_Data_config */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup Flash_Data_Private_define Flash_Data Private Define
 *  @{
 */


/*----------------------------------*/
/** 
 * @enum   Processing
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                              */
    
    PROC_START,             /*!< Start.                             */
    PROC_RAMTRANSFER,       /*!< RamTransfer.                       */
    PROC_ERASING,           /*!< Erasing.                           */
    PROC_REWRITING,         /*!< ReWriting.                         */
    
    PROC_AREA_ERASING,      /*!< Area Erasing.                      */
    PROC_PAGE_WRITING,      /*!< Page Writing.                      */
    PROC_PAGE_ERASING,      /*!< Page Erasing.                      */
    PROC_BLOCK_ERASING,     /*!< Block Erasing.                     */
    
    PROC_ERASE_ERROR,       /*!< Erase_Error.                       */
    PROC_WRITE_ERROR,       /*!< Write Error.                       */
    
    PROC_ERROR              /*!< Error.                             */
} Processing;

/** 
 *  @}
 */ /* End of group Flash_Data_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup Flash_Data_Private_variables Flash_Data Private Variables
 *  @{
 */
static Processing proc;                         /*!< Processing mode.          */

/**
 *  @}
 */ /* End of group Flash_Data_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup Flash_Data_Private_fuctions Flash_Data Private Fuctions
 *  @{
 */
void _PROC_START(void);
void _PROC_AREA_ERASING(void);
void _PROC_PAGE_WRITING(void);
void _PROC_PAGE_ERASING(void);
void _PROC_BLOCK_ERASING(void);
void _PROC_ERASE_ERROR(void);
void _PROC_WRITE_ERROR(void);

/**
 *  @}
 */ /* End of group Flash_Data_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup Flash_Data_Exported_functions Flash_Data Exported Functions
 *  @{
 */
 
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*------------------------*/
    /* Port Setting           */
    /*------------------------*/
    /* Enable the peripheral clock of LED    */
    GPIO_LED_enable_fsys();
    /* Configure LED  */
    GPIO_LED_Config();
    
    /* Enable the peripheral clock of SW     */
    GPIO_SW_enable_fsys();
    /* Configure SW  */
    GPIO_SW_Config();
    
    /* enable INT17a */
    INT17a_enable_IMC061();
    /* enable IHOSC1 */
    enable_IHOSC1();
    /* Internal HOSC Warm-up Time Set*/
    Set_CG_IOSC_WUPT_value();
    /* Warm-up Timer Start  */
    Set_param_CG_WUON();
    /* Check Warm-up Timer status */
    /* [CGWUPHCR]<WUEF>  : Read(wait for done) */
    Get_param_CG_WUEF();

    /* Enable the peripheral clock of DNF_B  */
    DNF_enable_fsys();
    /* Configure DNF_B  */
    DNF_B_Config();

    /*------------------------*/
    /* Processing Initial mode*/
    /*------------------------*/
    proc = PROC_START;
    
    /*------------------------*/
    /*          Loop          */
    /*------------------------*/
    while(1){
        switch(proc){
            case PROC_START:            _PROC_START();          break;
            case PROC_IDLE:                                     break;
            case PROC_AREA_ERASING:     _PROC_AREA_ERASING();   break;
            case PROC_PAGE_WRITING:     _PROC_PAGE_WRITING();   break;
            case PROC_PAGE_ERASING:     _PROC_PAGE_ERASING();   break;
            case PROC_BLOCK_ERASING:    _PROC_BLOCK_ERASING();  break;
            case PROC_ERASE_ERROR:      _PROC_ERASE_ERROR();    break;
            case PROC_WRITE_ERROR:      _PROC_WRITE_ERROR();    break;
            case PROC_ERROR:                                    break;
            default:                     proc = PROC_ERROR;     break;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_START.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_START(void)
{
    GPIO_LED1_ON();
    GPIO_LED2_OFF();
    GPIO_LED3_OFF();
    GPIO_LED4_OFF();
    
    NVIC_EnableIRQ( INT17_IRQn );
    
    proc = PROC_IDLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_AREA_ERASING.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_AREA_ERASING(void)
{
    if( fc_erase_area_data_flash( CMF_DATA_AREA_ALL ) == TXZ_SUCCESS )
    {
        GPIO_LED1_OFF();
        GPIO_LED2_OFF();
        GPIO_LED3_OFF();
        GPIO_LED4_OFF();
    
        proc = PROC_PAGE_WRITING;
    }
    else
    {
        proc = PROC_ERASE_ERROR;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_PAGE_WRITING.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_PAGE_WRITING(void)
{
    if( fc_write_page_data_flash(CMF_DATA_AREA_A , CMF_DATA_SIZE_A) == TXZ_SUCCESS )
    {
        if( fc_write_page_data_flash(CMF_DATA_AREA_B , CMF_DATA_SIZE_A) == TXZ_SUCCESS )
        {
            proc = PROC_PAGE_ERASING;
        }
        else
        {
            proc = PROC_WRITE_ERROR;
        }
    }
    else
    {
        proc = PROC_WRITE_ERROR;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_PAGE_ERASING.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_PAGE_ERASING(void)
{
    if( fc_erase_page_data_flash( CMF_DATA_AREA_A , 1 ) == TXZ_SUCCESS )
    {
        proc = PROC_BLOCK_ERASING;
    }
    else
    {
        proc = PROC_ERASE_ERROR;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_BLOCK_ERASING.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_BLOCK_ERASING(void)
{
    if( fc_erase_block_data_flash( CMF_DATA_AREA_C , 1) == TXZ_SUCCESS )
    {
        proc = PROC_START;
    }
    else
    {
        proc = PROC_ERASE_ERROR;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_ERASE_ERROR.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_ERASE_ERROR(void)
{
    GPIO_LED1_OFF();
    GPIO_LED2_OFF();
    GPIO_LED3_ON();
    GPIO_LED4_OFF();
    
    NVIC_DisableIRQ( INT17_IRQn );
    
    proc = PROC_ERROR;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_WRITE_ERROR.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_WRITE_ERROR(void)
{
    GPIO_LED1_OFF();
    GPIO_LED2_OFF();
    GPIO_LED3_OFF();
    GPIO_LED4_ON();
    
    NVIC_DisableIRQ( INT17_IRQn );
    
    proc = PROC_ERROR;
}

/*--------------------------------------------------*/
/** 
  * @brief  INT17 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   17:  Interrupt pin 17a/17b
  */
/*--------------------------------------------------*/
void INT17_IRQHandler(void)
{
    if(Get_param_INT157FLG())
    {
        Clear_flag_INTNCLR_IMC061();

        NVIC_DisableIRQ( INT17_IRQn );

        proc = PROC_AREA_ERASING;
    }
}

/**
 *  @}
 */ /* End of group Flash_Data_Exported_functions */

/**
 *  @}
 */ /* End of group Flash_Data */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
