/** 
 *******************************************************************************
 * @file    main.c
 * @brief   Flash Protect Sample Application.\n
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TSB_CMSIS_CORE.h"  /*!< Group Header file. */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cg.h"
#include "t32a.h"
#include "bsp.h"
#include "bsp_led.h"
#include "bsp_sw.h"

#include "except.h"
#include "flash.h"
#include "dnf.h"

/**
 *  @addtogroup Example Example
 *  @{
 */

/** 
 *  @addtogroup Flash_Protect Flash_Protect
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup Flash_Code_config Flash_Code Config
 *  @{
 */

#define    CMF_CODE_PAGE     FC_CODE_FLASH_PAGE3
#define    CMF_CODE_BLOCK    FC_CODE_FLASH_BLOCK2
const char CMF_Test_Data[] = "Execute data A";

/** 
 *  @}
 */ /* End of group Flash_Code_config */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup Flash_Protect_Private_define Flash_Protect Private Define
 *  @{
 */


/*----------------------------------*/
/** 
 * @enum   Processing
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                              */
    
    PROC_START,             /*!< Start.                             */
    PROC_RAMTRANSFER,       /*!< RAM Transfer.                      */
    PROC_ERASING,           /*!< Erasing.                           */
    PROC_REWRITING,         /*!< ReWriting.                         */
    
    PROC_AREA_ERASING,      /*!< Area Erasing.                      */
    PROC_PAGE_WRITING,      /*!< Page Writing.                      */
    PROC_PAGE_ERASING,      /*!< Page Erasing.                      */
    PROC_BLOCK_ERASING,     /*!< Block Erasing.                     */
    
    PROC_ERASE_ERROR,       /*!< Erase_Error.                       */
    PROC_WRITE_ERROR,       /*!< Write Error.                       */
    
    PROC_ERROR              /*!< Error.                             */
} Processing;

/** 
 *  @}
 */ /* End of group Flash_Protect_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup Flash_Protect_Private_variables Flash_Protect Private Variables
 *  @{
 */
static Processing proc;                         /*!< Processing mode.          */
static uint8_t    count;                        /*!<                           */
/**
 *  @}
 */ /* End of group Flash_Protect_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup Flash_Protect_Private_fuctions Flash_Protect Private Fuctions
 *  @{
 */
void _PROC_RAMTRANSFER(void);
void _PROC_PAGE_ERASING(void);
void _PROC_BLOCK_ERASING(void);
void _PROC_REWRITING(void);
void _PROC_ERASE_ERROR(void);
void _PROC_WRITE_ERROR(void);

/**
 *  @}
 */ /* End of group Flash_Protect_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup Flash_Protect_Exported_functions Flash_Protect Exported Functions
 *  @{
 */
 
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*------------------------*/
    /* Port Setting           */
    /*------------------------*/
    /* Enable the peripheral clock of LED    */
    GPIO_LED_enable_fsys();
    /* Configure LED  */
    GPIO_LED_Config();
    
    /* Enable the peripheral clock of SW     */
    GPIO_SW_enable_fsys();
    /* Configure SW  */
    GPIO_SW_Config();
    
    /* enable IHOSC1 */
    enable_IHOSC1();
    /* Internal HOSC Warm-up Time Set*/
    Set_CG_IOSC_WUPT_value();
    /* Warm-up Timer Start  */
    Set_param_CG_WUON();
    /* Check Warm-up Timer status */
    /* [CGWUPHCR]<WUEF>  : Read(wait for done) */
    Get_param_CG_WUEF();

    /* Enable the peripheral clock of DNF_B  */
    DNF_enable_fsys();
    /* Configure DNF_B  */
    DNF_B_Config();
    
    /*------------------------*/
    /* Processing Initial mode*/
    /*------------------------*/
    proc = PROC_RAMTRANSFER;
    
    /*------------------------*/
    /*          Loop          */
    /*------------------------*/
    while(1){
        switch(proc){
            case PROC_RAMTRANSFER:      _PROC_RAMTRANSFER();    break;
            case PROC_IDLE:                                     break;
            case PROC_PAGE_ERASING:     _PROC_PAGE_ERASING();   break;
            case PROC_BLOCK_ERASING:    _PROC_BLOCK_ERASING();  break;
            case PROC_REWRITING:        _PROC_REWRITING();      break;
            case PROC_ERASE_ERROR:      _PROC_ERASE_ERROR();    break;
            case PROC_WRITE_ERROR:      _PROC_WRITE_ERROR();    break;
            case PROC_ERROR:                                    break;
            default:                     proc = PROC_ERROR;     break;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_RAMTRANSFER.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_RAMTRANSFER(void)
{
    GPIO_LED1_ON();
    GPIO_LED2_ON();
    
    copy_fc_func();
    
    strcpy( (char*)DEMO_A_RAM , CMF_Test_Data );
    strcpy( (char*)DEMO_B_RAM , CMF_Test_Data );
    
    INT17a_enable_IMC061();
    
    NVIC_EnableIRQ( INT17_IRQn );
    
    proc = PROC_IDLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_PAGE_ERASING.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_PAGE_ERASING(void)
{
    GPIO_LED1_ON();
    GPIO_LED2_OFF();

    if( fc_protect_page_code_flash( CMF_CODE_PAGE ) == TXZ_SUCCESS )
    {
        if( fc_erase_page_code_flash( CMF_CODE_PAGE , 1 ) == TXZ_SUCCESS )
        {
            fc_protect_erase_code_flash();
        }
        else
        {
            if( fc_protect_erase_code_flash() == TXZ_SUCCESS )
            {
                if( fc_erase_page_code_flash( CMF_CODE_PAGE , 1 ) == TXZ_SUCCESS )
                {
                    proc = PROC_REWRITING;
                    return;
                }
            }
        }
    }

    GPIO_LED1_OFF();
    proc = PROC_ERASE_ERROR;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_BLOCK_ERASING.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_BLOCK_ERASING(void)
{
    GPIO_LED1_OFF();
    GPIO_LED2_ON();

    if( fc_protect_block_code_flash( CMF_CODE_BLOCK ) == TXZ_SUCCESS)
    {
        if( fc_erase_block_code_flash( CMF_CODE_BLOCK , 1 ) == TXZ_SUCCESS)
        {
            fc_protect_erase_code_flash();
        }
        else
        {
            if( fc_protect_erase_code_flash() == TXZ_SUCCESS)
            {
                if( fc_erase_block_code_flash( CMF_CODE_BLOCK , 1 ) == TXZ_SUCCESS)
                {
                    proc = PROC_REWRITING;
                    return;
                }
            }
        }
    }
    
    GPIO_LED2_OFF();

    proc = PROC_ERASE_ERROR;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_REWRITING.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_REWRITING(void)
{
    fc_erase_page_code_flash( CMF_CODE_PAGE , 1 );
    fc_erase_block_code_flash( CMF_CODE_BLOCK, 1 );

    if( rewrite_user_program(sizeof(CMF_Test_Data)) == TXZ_SUCCESS )
    {
        proc = PROC_RAMTRANSFER;
    }
    else
    {
        proc = PROC_WRITE_ERROR;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_ERASE_ERROR.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_ERASE_ERROR(void)
{
    GPIO_LED3_ON();
    
    _PROC_REWRITING();
    
    proc = PROC_ERROR;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_WRITE_ERROR.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_WRITE_ERROR(void)
{
    GPIO_LED1_OFF();
    GPIO_LED2_OFF();
    GPIO_LED3_OFF();
    GPIO_LED4_ON();
    
    proc = PROC_ERROR;
}

/*--------------------------------------------------*/
/** 
  * @brief  INT17 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   17:  Interrupt pin 17a/17b
  */
/*--------------------------------------------------*/
void INT17_IRQHandler(void)
{
    if( Get_param_INT157FLG() )
    {
        Clear_flag_INTNCLR_IMC061();

        NVIC_DisableIRQ( INT17_IRQn );

        if( count == 0 )
        {
            proc = PROC_PAGE_ERASING;
            count++;
        }
        else
        {
            proc = PROC_BLOCK_ERASING;
            count = 0;
        }
    }
}

/**
 *  @}
 */ /* End of group Flash_Protect_Exported_functions */

/**
 *  @}
 */ /* End of group Flash_Protect */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
