/** 
 *******************************************************************************
 * @file    main.c
 * @brief   I2C Master Trans Sample Application.\n
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TSB_CMSIS_CORE.h"  /*!< Group Header file. */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cg.h"
#include "t32a.h"
#include "bsp.h"
#include "bsp_led.h"
#include "bsp_sw.h"
#include "bsp_i2c.h"

#include "except.h"
#include "i2c.h"


/**
 *  @addtogroup Example Example
 *  @{
 */

/** 
 *  @addtogroup I2C_MasterTrans I2C_MasterTrans
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_MasterTrans_config I2C_MasterTrans Config
 *  @{
 */

#define MCU_NAME        "TMPM4KNFYA"

/** 
 *  @}
 */ /* End of group I2C_MasterTrans_config */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_MasterSlave_Private_define I2C_MasterSlave Private Define
 *  @{
 */


/*----------------------------------*/
/** 
 * @enum   Processing
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                              */
    
    PROC_START,             /*!< Start.                             */
    PROC_SET,               /*!< Set.                               */
    PROC_TRANS,             /*!< Trans.                             */
    PROC_FINISHED,          /*!< Finished.                          */
    PROC_TRANS_ERROR,       /*!< Trans Error.                       */
    
    PROC_ERROR              /*!< Error.                             */
} Processing;

/** 
 *  @}
 */ /* End of group I2C_MasterSlave_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_MasterTrans_Private_variables I2C_MasterTrans Private Variables
 *  @{
 */
#define DATA_LENGTH     (16)                    /*!< Length of base data.       */

static Processing proc;                         /*!< Processing mode.           */
static uint32_t   tx_length;                    /*!< Length of Master Tx Data.  */
static uint8_t    tx_buf[DATA_LENGTH];          /*!< Tx Data of Master.         */

/**
 *  @}
 */ /* End of group I2C_MasterTrans_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_MasterTrans_Private_fuctions I2C_MasterTrans Private Fuctions
 *  @{
 */
void _PROC_START(void);
void _PROC_SET(void);
void _PROC_FINISHED(void);
void _PROC_TRANS_ERROR(void);

/**
 *  @}
 */ /* End of group I2C_MasterTrans_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_MasterTrans_Exported_functions I2C_MasterTrans Exported Functions
 *  @{
 */
 
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*------------------------*/
    /* Port Setting           */
    /*------------------------*/
    /* Enable the peripheral clock of LED    */
    GPIO_LED_enable_fsys();
    /* Configure LED  */
    GPIO_LED_Config();
    
    /* Enable the peripheral clock of SW     */
    GPIO_SW_enable_fsys();
    /* Configure SW  */
    GPIO_SW_Config();
    
    /* Enable the peripheral clock of I2C    */
    GPIO_I2C_CH1_enable_fsys();
    /* Configure I2C1_Master  */
    GPIO_I2C1_Master_Config();
    
    /*------------------------*/
    /* I2C Master Setting     */
    /*------------------------*/
    /* Clock suply for I2C ch.1  */
    I2C_CH1_enable_fsys();
    /* Configure I2C1_Master     */
    I2C1_Master_Config();
    
    /*------------------------*/
    /* LED Initial Setting    */
    /*------------------------*/
    GPIO_LED1_OFF();
    GPIO_LED3_OFF();
    
    /*------------------------*/
    /* Processing Initial mode*/
    /*------------------------*/
    proc = PROC_START;
    
    /*------------------------*/
    /* Send Data Setting      */
    /*------------------------*/
    strcpy( (char *)tx_buf , MCU_NAME );
    
    /*------------------------*/
    /*          Loop          */
    /*------------------------*/
    while(1){
        switch(proc){
            case PROC_START:            _PROC_START();          break;
            case PROC_IDLE:                                     break;
            case PROC_SET:              _PROC_SET();            break;
            case PROC_TRANS:                                    break;
            case PROC_FINISHED:         _PROC_FINISHED();       break;
            case PROC_TRANS_ERROR:      _PROC_TRANS_ERROR();    break;
            case PROC_ERROR:                                    break;
            default:                     proc = PROC_ERROR;     break;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_START.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_START(void)
{
    INT17a_enable_IMC061();
    
    NVIC_EnableIRQ( INT17_IRQn );
    
    proc = PROC_IDLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_SET.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_SET(void)
{
    GPIO_LED3_OFF();
    
    NVIC_EnableIRQ( INTI2C1NST_IRQn );
    
    I2C1_start_condition( true );
    
    proc = PROC_TRANS;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_FINISHED.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_FINISHED(void)
{
    tx_length = 0;
    GPIO_LED1_FLIP();
    GPIO_LED3_OFF();
    
    NVIC_EnableIRQ( INTI2C1NST_IRQn );
    NVIC_EnableIRQ( INT17_IRQn );
    
    proc = PROC_START;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_TRANS_ERROR.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_TRANS_ERROR(void)
{
    tx_length = 0;
    I2C1_stop_condition();
    
    NVIC_EnableIRQ( INTI2C1NST_IRQn );
    
    GPIO_LED3_ON();
    
    NVIC_EnableIRQ( INT17_IRQn );
    
    proc = PROC_START;
}

/*--------------------------------------------------*/
/** 
  * @brief  INTI2C1NST IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note 
  */
/*--------------------------------------------------*/
void INTI2C1NST_IRQHandler(void)
{
    if( Get_param_I2C1_I2C() == FLAG_DETECT )
    {
        Clear_flag_I2C1_I2C();

        if( Get_param_I2C1_NACK() == FLAG_NOT_DETECT )
        {
            if( Get_param_I2C1_LRB() == LRB_0 )
            {
                Set_I2C1_DBR_value( tx_buf[tx_length] );
                tx_length++;

                if( tx_length >= DATA_LENGTH)
                {
                    proc = PROC_FINISHED;
                    I2C1_stop_condition();
                }
            }
            else
            {
                proc = PROC_TRANS_ERROR;
            }
        }
        else
        {
            Clear_flag_I2C1_NACK();
            proc = PROC_TRANS_ERROR;
        }
    }
    else
    {
        proc = PROC_ERROR;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  INT17 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   17:  Interrupt pin 17a/17b
  */
/*--------------------------------------------------*/
void INT17_IRQHandler(void)
{
    Clear_flag_INTNCLR_IMC061();
    
    NVIC_DisableIRQ( INT17_IRQn );
    
    proc = PROC_SET;
}

/**
 *  @}
 */ /* End of group I2C_MasterTrans_Exported_functions */

/**
 *  @}
 */ /* End of group I2C_MasterTrans */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
