/** 
 *******************************************************************************
 * @file    main.c
 * @brief   LVD_Interrupt Sample Application.\n
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TSB_CMSIS_CORE.h"  /*!< Group Header file. */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cg.h"
#include "lvd.h"
#include "t32a.h"
#include "bsp.h"
#include "bsp_led.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup LVD LVD Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup LVD_Private_macro LVD Private Macro
 *  @{
 */
/** 
 *  @addtogroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_LED_BLINK_CYCLE ((uint32_t)(1000))
#define CFG_LED_BLINK_FRQ   ((uint32_t)(2))
#define CFG_LED_BLINK_DUTY  (0.5)
#define CFG_LED_ON_TIME     ((uint32_t)(CFG_LED_BLINK_CYCLE / CFG_LED_BLINK_FRQ * CFG_LED_BLINK_DUTY))
#define CFG_LED_OFF_TIME    ((uint32_t)(CFG_LED_BLINK_CYCLE / CFG_LED_BLINK_FRQ * (1 - CFG_LED_BLINK_DUTY)))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */

/** 
 *  @addtogroup LVD_VOLTAGE LVD Detect Voltage Configuration
 *  @brief    LED Blink Interval Time(Interrupt Cycle).
 *  @{
 */
#define LVD_VOLTAGE_SEL     LVD_VOLTAGE_40


#define LVD_VOLTAGE_26      ((uint32_t)0x00000000)          /*!< LVD voltage 2.6+-0.1V    */
#define LVD_VOLTAGE_27      ((uint32_t)0x00000010)          /*!< LVD voltage 2.7+-0.1V    */
#define LVD_VOLTAGE_28      ((uint32_t)0x00000020)          /*!< LVD voltage 2.8+-0.1V    */
#define LVD_VOLTAGE_29      ((uint32_t)0x00000030)          /*!< LVD voltage 2.9+-0.1V    */
#define LVD_VOLTAGE_40      ((uint32_t)0x00000040)          /*!< LVD voltage 4.0+-0.1V    */
#define LVD_VOLTAGE_42      ((uint32_t)0x00000050)          /*!< LVD voltage 4.2+-0.1V    */
#define LVD_VOLTAGE_44      ((uint32_t)0x00000060)          /*!< LVD voltage 4.4+-0.1V    */
#define LVD_VOLTAGE_46      ((uint32_t)0x00000070)          /*!< LVD voltage 4.6+-0.1V    */
/** 
 *  @}
 */ /* End of group LVD_VOLTAGE */

/** 
 *  @}
 */ /* End of group LVD_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup LVD_Private_variables LVD Private Variables
 *  @{
 */
static uint32_t counter  = CFG_LED_ON_TIME;    /* Blink Time Counter         */
uint32_t lvd_status_now;                          /*!< LVD status now           */
uint32_t lvd_status_old;             /*!< LVD status old           */
uint32_t lvd_status_count = 0;                    /*!< LVD status count         */
uint32_t lvd_status = LVD_UNKNOWN;                  /*!< LVD status               */
volatile uint32_t lvd_led_pattern = 0;            /*!< LVD LED pattern          */
static bool led1_on_off = false;                 /*!< Led1 blink state         */
uint32_t led1_on_time;                           /*!< Led1 on time             */
uint32_t led1_off_time;                          /*!< Led1 on time             */
/**
 *  @}
 */ /* End of group LVD_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup LVD_Private_fuctions LVD Private Fuctions
 *  @{
 */

/**
 *  @}
 */ /* End of group LVD_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup LVD_Exported_functions LVD Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    uint32_t res = LVD_ERROR;

    /*------------------------*/
    /* Port Setting           */
    /*------------------------*/
    GPIO_LED_enable_fsys();
    GPIO_LED_Config();

    LVD_Config( LVD_VOLTAGE_SEL );
    LVD_start();

    T32A_CH0_enable_fsys();
    T32A0_A_IntervalTimer_Config();
    NVIC_EnableIRQ( INTT32A00AC_IRQn );
    T32A0_A_Start();

    for(;;){
        /*----------------------*/
        /* LVD Status Check            */
        /*----------------------*/
        lvd_status_now = Get_param_LVD_ST();

        if(lvd_status_now == lvd_status_old)
        {
            if(lvd_status_count++ >= LVD_STATUS_CT)
            {
                if(lvd_status != lvd_status_now)
                {
                    lvd_status = lvd_status_now;
                    res = LVD_SUCCESS;
                }
                lvd_status_count = 0;
            }
        }
        else
        {
            lvd_status_old = lvd_status_now;
            lvd_status_count = 0;
        }
        if(res == LVD_SUCCESS)
        {
            if(lvd_status == LVD_UPPER)
            {
                lvd_led_pattern = 0;
            }
            else
            {
                lvd_led_pattern = 1;
            }
        }
        else
        {
            /* do nothing */
        }
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  INTT32A00AC IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   95: T32A ch1 TimmerA/C compare match detection / Over flow / under flow
  */
/*--------------------------------------------------*/
void INTT32A00AC_IRQHandler(void)
{
    uint32_t t32a_status;

    t32a_status = Get_param_T32A0_INTA1();
    if( t32a_status == T32AxSTA_INTA1_MASK )
    {
        Clear_flag_T32A0_INTA();
        if( counter-- == 0 )
        {
            counter = CFG_LED_BLINK_CYCLE;
            if( lvd_led_pattern == 0 )
            {
                GPIO_LED2_ON();
                GPIO_LED1_OFF();
            }
            else
            {
                if( led1_on_off == false )
                {
                    counter = CFG_LED_ON_TIME;
                    led1_on_off = true;
                    GPIO_LED1_ON();
                }
                else
                {
                    counter = CFG_LED_OFF_TIME;
                    led1_on_off = false;
                    GPIO_LED1_OFF();
                }

                GPIO_LED2_OFF();
            }
        }
        else
        {
            /* no processing */
        }
    }
}

/**
 *  @}
 */ /* End of group LVD_Exported_functions */

/**
 *  @}
 */ /* End of group LVD */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
