/** 
 *******************************************************************************
 * @file    main.c
 * @brief   RAMP Sample Application.\n
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TSB_CMSIS_CORE.h"  /*!< Group Header file. */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cg.h"
#include "rpar.h"
#include "siwdt.h"
#include "t32a.h"
#include "bsp.h"
#include "uart_io.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup RAMP RAMP Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup RAMP_Private_macro RAMP Private Macro
 *  @{
 */
 
/** 
 *  @addtogroup DataLengtheConfiguration Data Length Configuration
 *  @brief    Data Length Configuration.
 *  @{
 */
#define CFG_DATA_LENGTH            ((uint32_t)10)
#define UART_IO_SEND_MAX           ((uint32_t)32)
#define UART_IO_RECEIVE_MAX        ((uint32_t)32)
/** 
 *  @}
 */ /* End of group DataLengtheConfiguration */


/** 
 *  @}
 */ /* End of group RAMP_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/*----------------------------------*/
/** 
 * @enum   Processing
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                        */
    PROC_OUTPUT_MSG,        /*!< Output first message.        */
    PROC_INPUT,             /*!< Wait to input.               */
    PROC_COMMAND_PARSE,     /*!< Output Echo back data.       */
    PROC_WRITE_EXEC,        /*!< Output Echo back data.       */
    PROC_READ_EXEC,         /*!< Output Echo back data.       */
    PROC_COMMAND_ERROR,        /*!< Output input error message.  */
    PROC_ERROR,             /*!< Error.                       */
    PROC_PARAM_ERROR,       /*!< Parameter Error.             */
    PROC_PARITY_ERROR         /*!< Parity Error.                */
} Processing;


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/*----------------------------------*/
/** 
 * @struct input_t
 * @brief  Input data information.
*/
/*----------------------------------*/
typedef struct
{
    char    data[(UART_IO_RECEIVE_MAX+1)];  /*!< Input data(+ for NULL 1byte). */
    uint8_t wp;                             /*!< Write pointer.                */
} input_t;

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup RAMP_Private_variables RAMP Private Variables
 *  @{
 */

uint32_t cfg_data_length = CFG_DATA_LENGTH;

static Processing proc;         /*!< Processing mode.        */
static input_t    input;        /*!< Input data information. */
static uint8_t write_data[CFG_DATA_LENGTH]; /*!< Write data buffer. */
uint8_t ram1[CFG_DATA_LENGTH] __attribute__((section(".ARM.__at_0x20001000")));
bool write_done = false;        /*!< Write Done */


/**
 *  @}
 */ /* End of group RAMP_Private_variables */



/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup RAMP_Private_fuctions RAMP Private Fuctions
 *  @{
 */


/*--------------------------------------------------*/
/** 
  * @brief  Clear input data.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void clear_data(void)
{
    uint8_t i;

    for( i = 0; i < (UART_IO_RECEIVE_MAX + 1); i++ )
    {
        input.data[i] = 0;
    }
    input.wp = 0;
}
/*--------------------------------------------------*/
/** 
  * @brief  Clear data.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void write_data_clear(void)
{
    uint8_t i;
    for( i = 0; i < CFG_DATA_LENGTH; i++ )
    {
        write_data[i] = 0;
    }
}


/**
 *  @}
 */ /* End of group RAMP_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup RAMP_Exported_functions RAMP Exported Functions
 *  @{
 */
 
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    int num;
    uint32_t status = 0;

    /*------------------------*/
    /* Port Setting           */
    /*------------------------*/
    uart_io_initialize();
    /*------------------------*/
    /* RAMP Setting           */
    /*------------------------*/
    RAMP_CH0_enable_fsys();
    RAMP0_Config();
    NVIC_EnableIRQ( INTPARI0_IRQn );

    proc = PROC_OUTPUT_MSG;
    /*------------------------*/
    /*          Loop          */
    /*------------------------*/ 
    while(1)
    {
        switch(proc)
        {
            /*--- Output "First Message" ---*/
            case PROC_OUTPUT_MSG:
            {
                /* Clear input data.     */
                clear_data();
                write_data_clear();

                /* Output first message. */
                if ( printf("command > ") != -1 )
                {
                    proc = PROC_INPUT;
                }
                else
                {
                    proc = PROC_ERROR;
                }
            }
            break;

             /*--- Wait to Input ---*/
            case PROC_INPUT:
            {
                int data;

                data = getchar();
                if( ( data != -1 ) && ( input.wp < UART_IO_RECEIVE_MAX ) )
                {
                    input.data[input.wp] = (char)data;
                    input.wp++;
                    switch ((char)data)
                    {
                        /*--- LF: line feed ---*/
                        case '\n':
                            proc = PROC_COMMAND_PARSE;
                            break;
                        /*--- default ---*/
                        default:
                            /* no process */
                            break;
                    }
                }
                else
                {
                    proc = PROC_COMMAND_ERROR;
                }
            }
            break;

            /*--- Output "Echo Back Data" ---*/
            case PROC_COMMAND_PARSE:
            {
                int i;
                if( ( strncmp((const char *)&input.data[0], "write", 5 ) ) == 0 )
                {
                    num = input.wp - 6; /* len - (write (5charactors) + space(1 caractors)) */
                    if (num>CFG_DATA_LENGTH+1){
                        proc = PROC_PARAM_ERROR;
                        break;
                    }
                    for( i = 0; i < CFG_DATA_LENGTH; i++ )
                    {
                        write_data[i] = (uint8_t)input.data[6 + i];
                    }
                    if( printf("\n") == -1 )
                    {
                        proc = PROC_ERROR;
                    }
                    if( printf("write data > %s\n", (const char *)&write_data[0])  == -1 )
                    {
                        proc = PROC_ERROR;
                    }
                    proc = PROC_WRITE_EXEC;
                    break;
                }
                else if( ( strncmp((const char *)&input.data[0], "read", 4 ) ) == 0 )
                {
                    if( write_done == false )
                    {
                        proc = PROC_ERROR;
                        break;
                    }
                    /* RAM AREA Reading */
                    if( printf("\n") == -1 )
                    {
                        proc = PROC_ERROR;
                    }
                    if( printf("read data > ") == -1 )
                    {
                        proc = PROC_ERROR;
                    }
                    if( printf("%s", (const char*)&ram1[0]) == -1 )
                    {
                        proc = PROC_ERROR;
                    }
                    if( printf("\n") == -1 )
                    {
                        proc = PROC_ERROR;
                    }
                    if ( proc != PROC_ERROR )
                    {
                        status = Get_param_RPAR_ST();
                        if( status != 0 )
                        {
                            proc = PROC_PARITY_ERROR;
                        }
                        else
                        {
                            proc = PROC_OUTPUT_MSG;
                        }
                    }
                }
                else
                {
                    proc = PROC_COMMAND_ERROR;
                }
            }
            break;

            case PROC_WRITE_EXEC:
            {
                /* RAM AREA writing */
                if( strncpy((char *)ram1, (const char *)write_data,CFG_DATA_LENGTH ) == NULL )
                {
                    proc = PROC_COMMAND_ERROR;
                }
                status = Get_param_RPAR_ST();
                if( status != 0 )
                {
                    proc = PROC_PARITY_ERROR;
                }
                else
                {
                    /* write_data CLEAR */
                    write_data_clear();
                    proc = PROC_OUTPUT_MSG;
                }
                write_done = true;
            }
            break;

            /*--- Output "Input Error Message" ---*/
            case PROC_COMMAND_ERROR:
            {
                if( printf("\n Command Error !! \n") != -1 )
                {
                    proc = PROC_OUTPUT_MSG;
                }
                else
                {
                    proc = PROC_ERROR;
                }
            }
            break;

            /*--- Output "Parameter Error Message" ---*/
            case PROC_PARAM_ERROR:
            {
                if( printf("\n Parameter Error !! \n") != -1 )
                {
                    proc = PROC_OUTPUT_MSG;
                }
                else
                {
                    proc = PROC_ERROR;
                }
            }
            break;

            /*--- Output "Parity Error Message" ---*/
            case PROC_PARITY_ERROR:
            {
                uint32_t area_num;
                area_num = status & 0x000000ff;
                if( printf("\n RAM AREA %d Parity Error !! \n", area_num ) != -1 )
                {
                    proc = PROC_OUTPUT_MSG;
                }
                else
                {
                    proc = PROC_ERROR;
                }
            }
            break;

            /*--- Error ---*/
            case PROC_ERROR:
                /* System Error. Please Debug. */
                break;
            /*--- Default ---*/
            default:
                proc = PROC_ERROR;
             break;
        }
    }

}


/*--------------------------------------------------*/
/** 
  * @brief  INTPARI0 Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void INTPARI0_IRQHandler(void)
{
    uint32_t rparst = Get_param_RPAR_ST();

    proc = PROC_PARITY_ERROR;

}

/**
 *  @}
 */ /* End of group RAMP_Exported_functions */

/**
 *  @}
 */ /* End of group RAMP */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
