/** 
 *******************************************************************************
 * @file    main.c
 * @brief   TSPI Master Receive Sample Application.\n
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TSB_CMSIS_CORE.h"  /*!< Group Header file. */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cg.h"
#include "t32a.h"
#include "bsp.h"
#include "bsp_led.h"
#include "bsp_sw.h"
#include "bsp_tspi.h"

#include "except.h"
#include "crc.h"
#include "tspi.h"
#include "tspi_dma.h"
#include "dnf.h"
#include "sys_timer.h"
     
/**
 *  @addtogroup Example Example
 *  @{
 */

/** 
 *  @addtogroup TSPI_MasterReceive TSPI_MasterReceive
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSPI_MasterReceive_config TSPI_MasterReceive Config
 *  @{
 */
#define MCU_NAME        "TMPM4KNFYA"


#define CHK_CODE_CRC16  16
#define CHK_CODE_CRC32  32
#define CHK_CODE        CHK_CODE_CRC16


#if CHK_CODE == CHK_CODE_CRC16
    #define DATA_LENGTH     (14)                    /*!< Length of base data.      */
#else
    #define DATA_LENGTH     (16)                    /*!< Length of base data.      */
#endif


#define FIFO_MODE        0
#define DMA_MODE         1
#define RX_MODE          FIFO_MODE


#define RX_FILL_LEVEL    4

/** 
 *  @}
 */ /* End of group TSPI_MasterReceive_config */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSPI_MasterSlave_Private_define TSPI_MasterSlave Private Define
 *  @{
 */


/*----------------------------------*/
/** 
 * @enum   Processing
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
#if RX_MODE  == FIFO_MODE
    PROC_FIFO_START,        /*!< Start.                             */
    PROC_FIFO_IDLE,         /*!< Idle.                              */
    PROC_FIFO_SET,          /*!< Set.                               */
    PROC_FIFO_TRANS,        /*!< Trans.                             */
    PROC_FIFO_RECV,         /*!< Receive.                           */
    PROC_FIFO_FINISHED,     /*!< Finished.                          */
    PROC_FIFO_CMP,          /*!< Compare.                           */
    PROC_FIFO_ERROR         /*!< Error.                             */
#else
    PROC_DMAC_START,        /*!< Start.                             */
    PROC_DMAC_IDLE,         /*!< Idle.                              */
    PROC_DMAC_SET,          /*!< Set.                               */
    PROC_DMAC_TRANS,        /*!< Trans.                             */
    PROC_DMAC_RECV,         /*!< Receive.                           */
    PROC_DMAC_FINISHED,     /*!< Finished.                          */
    PROC_DMAC_CMP,          /*!< Compare.                           */
    PROC_DMAC_ERROR,        /*!< Error.                             */
    PROC_DMAC_WAIT,         /*!< Wait.                              */
    PROC_DMAC_OVER          /*!< Over.                              */
#endif
} Processing;

/** 
 *  @}
 */ /* End of group TSPI_MasterSlave_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSPI_MasterReceive_Private_variables TSPI_MasterReceive Private Variables
 *  @{
 */

static Processing proc;                         /*!< Processing mode.           */
#if RX_MODE  == FIFO_MODE
    static uint8_t  rx_length;                  /*!< Length of Master Rx Data.  */
#endif
    static uint8_t  rx_buf[DATA_LENGTH];        /*!< Rx Data of Master.         */

uint8_t     sw_cnt;
/**
 *  @}
 */ /* End of group TSPI_MasterReceive_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSPI_MasterReceive_Private_fuctions TSPI_MasterReceive Private Fuctions
 *  @{
 */
void MASTER_Receive(void);
uint32_t Calc_CRC_data(uint8_t *buf);

#if RX_MODE  == FIFO_MODE
void _PROC_FIFO_START(void);
void _PROC_FIFO_SET(void);
void _PROC_FIFO_FINISHED(void);
void _PROC_FIFO_CMP(void);
void _PROC_FIFO_ERROR(void);
#else
void _PROC_DMAC_START(void);
void _PROC_DMAC_SET(void);
void _PROC_DMAC_FINISHED(void);
void _PROC_DMAC_CMP(void);
void _PROC_DMAC_ERROR(void);
#endif

/**
 *  @}
 */ /* End of group TSPI_MasterReceive_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
uint8_t rx_data[DATA_LENGTH];
/** 
 *  @addtogroup TSPI_MasterReceive_Exported_functions TSPI_MasterReceive Exported Functions
 *  @{
 */
 
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*------------------------*/
    /* Port Setting           */
    /*------------------------*/
    /* Enable the peripheral clock of LED    */
    GPIO_LED_enable_fsys();
    /* Configure LED  */
    GPIO_LED_Config();
    
    /* Enable the peripheral clock of SW    */
    GPIO_SW_enable_fsys();
    /* Configure SW  */
    GPIO_SW_Config();
    
    /* Enable the peripheral clock of TSPI port */
    GPIO_TSPI_CH0_enable_fsys();
    /* Configure TSPI0_Master port */
    GPIO_TSPI0_Master_Config();
    
    /*------------------------*/
    /* CRC Initial Setting    */
    /*------------------------*/
    /* Enable CRC             */
    CRC_enable_fsys();

    /* Enable the peripheral clock of DNF */
    DNF_enable_fsys();
    /* Enable DNF */
    DNF_B_Config();

    /* Enable the peripheral clock of TSPI ch.0 */
    TSPI_CH0_enable_fsys();
    /* Configure TSPI0 Master Receive */
    TSPI0_Master_Config(( TSPIxCR1_MASTER_RX_INIT | ((uint32_t)DATA_LENGTH & TSPIxCR1_FC_MASK))
                       ,( TSPIxCR2_MASTER_RX_INIT | ( RX_FILL_LEVEL << 8 )));
    
#if RX_MODE  == FIFO_MODE
    proc = PROC_FIFO_START;
#else
    proc = PROC_DMAC_START;
#endif
    /*------------------------*/
    /*          Loop          */
    /*------------------------*/
    while(1){
        MASTER_Receive();
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  INT17 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   17:  Interrupt pin 17a/17b
  */
/*--------------------------------------------------*/
void INT17_IRQHandler(void)
{
    if(Get_param_INT157FLG())
    {
        Clear_flag_INTNCLR_IMC061();

#if RX_MODE  == FIFO_MODE
        if(proc ==  PROC_FIFO_IDLE)
#else
        if(proc ==  PROC_DMAC_IDLE)
#endif
        {
            NVIC_DisableIRQ( INT17_IRQn );

            GPIO_LED3_OFF();
            GPIO_LED4_OFF();

#if RX_MODE  == FIFO_MODE
            proc = PROC_FIFO_SET;

            rx_length = 0;
#else
            proc = PROC_DMAC_SET;
#endif
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  MASTER_Receive.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void MASTER_Receive(void)
{
    switch( proc )
    {
#if RX_MODE  == FIFO_MODE
        case PROC_FIFO_START:     _PROC_FIFO_START();     break;
        case PROC_FIFO_IDLE:                              break;
        case PROC_FIFO_SET:       _PROC_FIFO_SET();       break;
        case PROC_FIFO_RECV:                              break;
        case PROC_FIFO_FINISHED:  _PROC_FIFO_FINISHED();  break;
        case PROC_FIFO_CMP:       _PROC_FIFO_CMP();       break;
        case PROC_FIFO_ERROR:     _PROC_FIFO_ERROR();     break;
#else
        case PROC_DMAC_START:     _PROC_DMAC_START();     break;
        case PROC_DMAC_IDLE:                              break;
        case PROC_DMAC_SET:       _PROC_DMAC_SET();       break;
        case PROC_DMAC_RECV:                              break;
        case PROC_DMAC_FINISHED:  _PROC_DMAC_FINISHED();  break;
        case PROC_DMAC_CMP:       _PROC_DMAC_CMP();       break;
        case PROC_DMAC_ERROR:     _PROC_DMAC_ERROR();     break;
#endif
        default:
            break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Calc_CRC_data(uint8_t buf).
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t Calc_CRC_data(uint8_t *buf)
{
    uint32_t    crc_data = 0;
    uint8_t     data_size  = strlen((const char *)buf) + 1;

#if CHK_CODE == CHK_CODE_CRC16
    CRC_Config((uint32_t)(CRCTYP_CFMT_CRC16 | CRCTYP_DBIT_32BIT), (uint32_t)CRCCLC_CRC16_INIT);
#else
    CRC_Config((uint32_t)(CRCTYP_CFMT_CRC32 | CRCTYP_DBIT_32BIT), (uint32_t)CRCCLC_CRC32_INIT);
#endif

    CRC_calculation((uint32_t)buf, (uint32_t)data_size);
    crc_data = Get_CRC_CLC_value();

    return( crc_data );
}

#if RX_MODE  == FIFO_MODE
/* FIFO_MODE ================================================ */
/*--------------------------------------------------*/
/** 
  * @brief  _PROC_FIFO_START.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_FIFO_START(void)
{
    INT17a_enable_IMC061();
    
    NVIC_EnableIRQ( INT17_IRQn );
    
    Set_param_TSPI0_RFFLLCLR();
    
    proc = PROC_FIFO_IDLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_FIFO_SET.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_FIFO_SET(void)
{
    if( rx_length == 0 )
    {
        NVIC_EnableIRQ( INTSC0RX_IRQn );

        NVIC_EnableIRQ( INTSC0ERR_IRQn );

        enable_TSPI0_TRXE();

        proc =  PROC_FIFO_RECV;
    }
    else
    {
        rx_length = 0;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_FIFO_FINISHED.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_FIFO_FINISHED(void)
{
    if( Get_param_TSPI0_RFFLL() == FLAG_NOT_DETECT )
    {
        if( Get_param_TSPI0_RXRUN() == FLAG_NOT_DETECT )
        {
            disable_TSPI0_TRXE();
            Clear_flag_TSPI0_RXEND();
            Clear_flag_TSPI0_INTRXFF();
            NVIC_DisableIRQ( INTSC0RX_IRQn );
            NVIC_DisableIRQ( INTSC0ERR_IRQn );

            proc = PROC_FIFO_CMP;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_FIFO_CMP.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_FIFO_CMP(void)
{
    uint32_t    CRC_data;
    uint8_t     ck_buf[DATA_LENGTH];        /*!< Check Data                   */
    uint8_t     *mcu_name = &rx_buf[1];
    uint8_t     data_size = rx_buf[0];
    uint8_t     *wk_ptr;
    
    memset( ck_buf , 0 , DATA_LENGTH);
    
    if( data_size > DATA_LENGTH )
    {
        data_size = DATA_LENGTH;
    }
    strncpy( (char *)&ck_buf[0] , (char *)mcu_name, data_size );
    
    CRC_data = Calc_CRC_data( ck_buf );

#if CHK_CODE == CHK_CODE_CRC16
    wk_ptr = (uint8_t *)&CRC_data;
    ck_buf[data_size] = *wk_ptr++;
    ck_buf[data_size+1] = *wk_ptr++;
    data_size += 2;
#else
    wk_ptr = (uint8_t *)&CRC_data;
    ck_buf[data_size] = *wk_ptr++;
    ck_buf[data_size+1] = *wk_ptr++;
    ck_buf[data_size+2] = *wk_ptr++;
    ck_buf[data_size+3] = *wk_ptr++;
    data_size += 4;
#endif
    
    if( memcmp(&rx_buf[1] , ck_buf , data_size ) == 0 )
    {
        GPIO_LED1_FLIP();
    }
    else
    {
        GPIO_LED1_FLIP();
        GPIO_LED4_ON();
    }

    do
    {
        wait(10000);
        if( Get_PSW1_input() == BSP_PSW_OFF )
        {
            sw_cnt++;
        }
        else
        {
            sw_cnt = 0;
        }
    } while( sw_cnt < 10 );
    sw_cnt = 0;

    NVIC_ClearPendingIRQ( INT17_IRQn );
    Clear_flag_INTNCLR_IMC061();
    
    proc = PROC_FIFO_START;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_FIFO_ERROR.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_FIFO_ERROR(void)
{
    disable_TSPI0_TRXE();
    Clear_flag_TSPI0_RXEND();
    Clear_flag_TSPI0_INTRXFF();
    NVIC_DisableIRQ( INTSC0RX_IRQn );
    NVIC_DisableIRQ( INTSC0ERR_IRQn );
    
    rx_length = 0;
    
    proc = PROC_FIFO_START;
    
    GPIO_LED1_OFF();
    GPIO_LED3_ON();
}

/*--------------------------------------------------*/
/** 
  * @brief  INTSC0RX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   
  */
/*--------------------------------------------------*/
void INTSC0RX_IRQHandler(void)
{
    uint8_t i;
    if( proc == PROC_FIFO_RECV )
    {
        if( Get_param_TSPI0_INTRXFF() == FLAG_DETECT )
        {
            Clear_flag_TSPI0_INTRXFF();

            do
            {
                rx_buf[rx_length] = Get_TSPI0_DR_value();
                rx_length++;
                for( i = 0; i < 10; i++ ){;}      /* wait for data set FIFO to DR */
                if( rx_length >= DATA_LENGTH )
                {
                    disable_TSPI0_TRXE();
                    proc = PROC_FIFO_FINISHED;
                    return;
                }

            }while( Get_TSPI0_RLVL_value() != 0 );
        }
        else
        {
            if( Get_param_TSPI0_RXEND() == FLAG_DETECT )
            {
                Clear_flag_TSPI0_RXEND();
                do
                {
                    rx_buf[rx_length] = Get_TSPI0_DR_value();
                    rx_length++;
                    for( i = 0; i < 10; i++ ){;}  /* wait for data set FIFO to DR */
                    if( rx_length >= DATA_LENGTH )
                    {
                        disable_TSPI0_TRXE();
                        proc = PROC_FIFO_FINISHED;
                        return;
                    }
                }while( Get_TSPI0_RLVL_value() != 0 );
            }
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  INTSC0ERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   
  */
/*--------------------------------------------------*/
void INTSC0ERR_IRQHandler(void)
{
    Clear_allflag_TSPI0_ERR();
    
    proc = PROC_FIFO_ERROR;
}

#else
/* DMA_MODE ================================================= */
/*--------------------------------------------------*/
/** 
  * @brief  _PROC_DMAC_START.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_DMAC_START(void)
{
    DMAC_A_enable_fsys();
    DMA_Config();
    NVIC_EnableIRQ( INTDMAATC_IRQn );
    NVIC_EnableIRQ( INTDMAAERR_IRQn );

    INT17a_enable_IMC061();
    NVIC_EnableIRQ( INT17_IRQn );
    
    Set_param_TSPI0_RFFLLCLR();
    
    proc = PROC_DMAC_IDLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_DMAC_SET.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_DMAC_SET(void)
{

    if( TSPI_DMA_Receive( DATA_LENGTH ) )
    {
        memcpy( &rx_buf[0], &rx_data[0], DATA_LENGTH );
        proc = PROC_DMAC_FINISHED;
    }
    else
    {
        proc = PROC_DMAC_ERROR;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_DMAC_FINISHED.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_DMAC_FINISHED(void)
{
    disable_TSPI0_TRXE();
    
    NVIC_DisableIRQ( INTDMAATC_IRQn );
    NVIC_DisableIRQ( INTDMAAERR_IRQn );
    
    proc = PROC_DMAC_CMP;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_DMAC_CMP.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_DMAC_CMP(void)
{
    uint32_t   CRC_data;
    uint8_t    ck_buf[DATA_LENGTH];         /*!< Check Data                   */
    uint8_t    *mcu_name = &rx_buf[1];
    uint8_t    data_size = rx_buf[0];
    uint8_t     *wk_ptr;
    
    memset( ck_buf , 0 , DATA_LENGTH );
    
    if( data_size > DATA_LENGTH )
    {
        data_size = DATA_LENGTH;
    }
    strncpy( (char *)&ck_buf[0] , (char *)mcu_name, data_size );
    
    CRC_data = Calc_CRC_data(ck_buf);

#if CHK_CODE == CHK_CODE_CRC16
    wk_ptr = (uint8_t *)&CRC_data;
    ck_buf[data_size] = *wk_ptr++;
    ck_buf[data_size+1] = *wk_ptr++;
    data_size += 2;
#else
    wk_ptr = (uint8_t *)&CRC_data;
    ck_buf[data_size] = *wk_ptr++;
    ck_buf[data_size+1] = *wk_ptr++;
    ck_buf[data_size+2] = *wk_ptr++;
    ck_buf[data_size+3] = *wk_ptr++;
    data_size += 4;
#endif
    
    if( memcmp(&rx_buf[1] , ck_buf , data_size ) == 0 )
    {
        GPIO_LED1_FLIP();
    }
    else
    {
        GPIO_LED1_FLIP();
        GPIO_LED4_ON();
    }

    do
    {
        wait(10000);
        if( Get_PSW1_input() == BSP_PSW_OFF )
        {
            sw_cnt++;
        }
        else
        {
            sw_cnt = 0;
        }
    } while( sw_cnt < 10 );
    sw_cnt = 0;

    NVIC_ClearPendingIRQ( INT17_IRQn );
    Clear_flag_INTNCLR_IMC061();

    proc = PROC_DMAC_START;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_DMAC_ERROR.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_DMAC_ERROR(void)
{
    disable_TSPI0_TRXE();
    
    NVIC_DisableIRQ( INTDMAATC_IRQn );
    NVIC_DisableIRQ( INTDMAAERR_IRQn );
    
    proc = PROC_DMAC_START;
    
    GPIO_LED1_OFF();
    GPIO_LED3_ON();
}
#endif

/**
 *  @}
 */ /* End of group TSPI_MasterReceive_Exported_functions */

/**
 *  @}
 */ /* End of group TSPI_MasterReceive */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
