/** 
 *******************************************************************************
 * @file    main.c
 * @brief   TSPI Master Trans Sample Application.\n
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "TSB_CMSIS_CORE.h"  /*!< Group Header file. */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cg.h"
#include "t32a.h"
#include "bsp.h"
#include "bsp_led.h"
#include "bsp_sw.h"
#include "bsp_tspi.h"

#include "except.h"
#include "crc.h"
#include "tspi.h"
#include "tspi_dma.h"
#include "dnf.h"
#include "sys_timer.h"
     
/**
 *  @addtogroup Example Example
 *  @{
 */

/** 
 *  @addtogroup TSPI_MasterTrans TSPI_MasterTrans
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSPI_MasterTrans_config TSPI_MasterTrans Config
 *  @{
 */
#define MCU_NAME        "TMPM4KNFYA"


#define CHK_CODE_CRC16  16
#define CHK_CODE_CRC32  32
#define CHK_CODE        CHK_CODE_CRC16


#if CHK_CODE == CHK_CODE_CRC16
    #define DATA_LENGTH     (14)                    /*!< Length of base data.      */
#else
    #define DATA_LENGTH     (16)                    /*!< Length of base data.      */
#endif


#define FIFO_MODE        0
#define DMA_MODE         1
#define TX_MODE          FIFO_MODE


#define TX_FILL_LEVEL    3

/** 
 *  @}
 */ /* End of group TSPI_MasterTrans_config */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSPI_MasterSlave_Private_define TSPI_MasterSlave Private Define
 *  @{
 */


/*----------------------------------*/
/** 
 * @enum   Processing
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
#if TX_MODE  == FIFO_MODE
    PROC_FIFO_START,        /*!< Start.                             */
    PROC_FIFO_IDLE,         /*!< Idle.                              */
    PROC_FIFO_SET,          /*!< Set.                               */
    PROC_FIFO_TRANS,        /*!< Trans.                             */
    PROC_FIFO_RECV,         /*!< Receive.                           */
    PROC_FIFO_FINISHED,     /*!< Finished.                          */
    PROC_FIFO_CMP,          /*!< Compare.                           */
    PROC_FIFO_ERROR         /*!< Error.                             */
#else
    PROC_DMAC_START,        /*!< Start.                             */
    PROC_DMAC_IDLE,         /*!< Idle.                              */
    PROC_DMAC_SET,          /*!< Set.                               */
    PROC_DMAC_FINISHED,     /*!< Finished.                          */
    PROC_DMAC_ERROR,        /*!< Error.                             */
    PROC_DMAC_WAIT,         /*!< Wait.                              */
    PROC_DMAC_OVER          /*!< Over.                              */
#endif
} Processing;

/** 
 *  @}
 */ /* End of group TSPI_MasterSlave_Private_define */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSPI_MasterTrans_Private_variables TSPI_MasterTrans Private Variables
 *  @{
 */

static Processing proc;                         /*!< Processing mode.          */
#if TX_MODE == FIFO_MODE
static uint8_t      tx_length;                  /*!< Length of Master Tx Data. */
#endif
static uint8_t      tx_buf[DATA_LENGTH];        /*!< Tx Data of Master.        */

uint8_t     sw_cnt;
/**
 *  @}
 */ /* End of group TSPI_MasterTrans_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSPI_MasterTrans_Private_fuctions TSPI_MasterTrans Private Fuctions
 *  @{
 */
void MASTER_TRANS(void);
void TSPI_Calc_CRC_TXdata(void);
uint32_t  Calc_CRC_data(uint8_t *buf);

#if TX_MODE  == FIFO_MODE
void _PROC_FIFO_START(void);
void _PROC_FIFO_SET(void);
void _PROC_FIFO_FINISHED(void);
void _PROC_FIFO_ERROR(void);
#else
void _PROC_DMAC_START(void);
void _PROC_DMAC_SET(void);
void _PROC_DMAC_FINISHED(void);
void _PROC_DMAC_ERROR(void);
#endif

/**
 *  @}
 */ /* End of group TSPI_MasterTrans_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
uint8_t rx_data[DATA_LENGTH];
/** 
 *  @addtogroup TSPI_MasterTrans_Exported_functions TSPI_MasterTrans Exported Functions
 *  @{
 */
 
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*------------------------*/
    /* Port Setting           */
    /*------------------------*/
    /* Enable the peripheral clock of LED    */
    GPIO_LED_enable_fsys();
    /* Configure LED  */
    GPIO_LED_Config();
    
    /* Enable the peripheral clock of SW    */
    GPIO_SW_enable_fsys();
    /* Configure SW  */
    GPIO_SW_Config();
    
    /* Enable the peripheral clock of TSPI port  */
    GPIO_TSPI_CH0_enable_fsys();
    /* Configure TSPI0_Master  */
    GPIO_TSPI0_Master_Config();
    
    /*------------------------*/
    /* CRC Initial Setting    */
    /*------------------------*/
    /* Enable CRC             */
    CRC_enable_fsys();
    
    /*------------------------*/
    /* TXdata Setting          */
    /*------------------------*/
    TSPI_Calc_CRC_TXdata();
    
    /* Enable the peripehral clock of DNF */
    DNF_enable_fsys();
    /* Confiure DNF */
    DNF_B_Config();

    /* Enable the peripehral clock of TSPI ch.0 */
    TSPI_CH0_enable_fsys();
    /* Confiure TSPI Master mode */
    TSPI0_Master_Config(( TSPIxCR1_MASTER_TX_INIT | ((uint32_t)DATA_LENGTH & TSPIxCR1_FC_MASK))
                       ,( TSPIxCR2_MASTER_TX_INIT | (TX_FILL_LEVEL << 12)));
    
#if TX_MODE  == FIFO_MODE
    proc = PROC_FIFO_START;
#else
    proc = PROC_DMAC_START;
#endif
    /*------------------------*/
    /*          Loop          */
    /*------------------------*/
    while(1){
        MASTER_TRANS();
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  INT17 IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   17:  Interrupt pin 17a/17b
  */
/*--------------------------------------------------*/
void INT17_IRQHandler(void)
{
    if( Get_param_INT157FLG() )
    {
        Clear_flag_INTNCLR_IMC061();

#if TX_MODE  == FIFO_MODE
        if( proc ==  PROC_FIFO_IDLE )
#else
        if( proc ==  PROC_DMAC_IDLE )
#endif
        {
            NVIC_DisableIRQ( INT17_IRQn );

            GPIO_LED3_OFF();

#if TX_MODE  == FIFO_MODE
            proc = PROC_FIFO_SET;

            tx_length = 0;
#else
            proc = PROC_DMAC_SET;
#endif
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  TSPI_Calc_CRC_TXdata
  * @param  -
  * @return -
  * @retval -
  * @note   
  */
/*--------------------------------------------------*/
void TSPI_Calc_CRC_TXdata(void)
{
    uint32_t    CRC_data;
    uint8_t     mcu_name[] = MCU_NAME;
    uint8_t     data_size  = sizeof( mcu_name );
    uint8_t     *wk_ptr;
    
    memset( tx_buf , 0 , DATA_LENGTH );
    
    tx_buf[0] = data_size;
    strncpy( (char *)&tx_buf[1] , (char *)mcu_name, data_size );
    data_size += 1;
    
    CRC_data = Calc_CRC_data( mcu_name );

#if CHK_CODE == CHK_CODE_CRC16
    wk_ptr = (uint8_t *)&CRC_data;
    tx_buf[data_size] = *wk_ptr++;
    tx_buf[data_size+1] = *wk_ptr++;
#else
    wk_ptr = (uint8_t *)&CRC_data;
    tx_buf[data_size] = *wk_ptr++;
    tx_buf[data_size+1] = *wk_ptr++;
    tx_buf[data_size+2] = *wk_ptr++;
    tx_buf[data_size+3] = *wk_ptr++;
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  MASTER_TRANS.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void MASTER_TRANS(void)
{
    switch( proc )
    {
#if TX_MODE  == FIFO_MODE
        case PROC_FIFO_START:       _PROC_FIFO_START();      break;
        case PROC_FIFO_IDLE:                                 break;
        case PROC_FIFO_SET:         _PROC_FIFO_SET();        break;
        case PROC_FIFO_TRANS:                                break;
        case PROC_FIFO_FINISHED:    _PROC_FIFO_FINISHED();   break;
        case PROC_FIFO_ERROR:       _PROC_FIFO_ERROR();      break;
#else
        case PROC_DMAC_START:       _PROC_DMAC_START();      break;
        case PROC_DMAC_IDLE:                                 break;
        case PROC_DMAC_SET:        _PROC_DMAC_SET();         break;
        case PROC_DMAC_FINISHED:   _PROC_DMAC_FINISHED();    break;
        case PROC_DMAC_ERROR:      _PROC_DMAC_ERROR();       break;
#endif
        default:
            break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Calc_CRC_data(uint8_t buf).
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t Calc_CRC_data(uint8_t *buf)
{
    uint32_t    crc_data = 0;
    uint8_t     data_size  = strlen((const char *)buf) + 1;

#if CHK_CODE == CHK_CODE_CRC16
    CRC_Config( (uint32_t)(CRCTYP_CFMT_CRC16 | CRCTYP_DBIT_32BIT), (uint32_t)CRCCLC_CRC16_INIT );
#else
    CRC_Config( (uint32_t)(CRCTYP_CFMT_CRC32 | CRCTYP_DBIT_32BIT), (uint32_t)CRCCLC_CRC32_INIT );
#endif

    CRC_calculation( (uint32_t)buf, (uint32_t)data_size );
    crc_data = Get_CRC_CLC_value();

    return( crc_data );
}

#if TX_MODE  == FIFO_MODE
/* FIFO_MODE ================================================ */
/*--------------------------------------------------*/
/** 
  * @brief  _PROC_FIFO_START.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_FIFO_START(void)
{
    INT17a_enable_IMC061();
    
    NVIC_EnableIRQ( INT17_IRQn );
    
    Set_param_TSPI0_TFEMPCLR();
    
    proc = PROC_FIFO_IDLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_FIFO_SET.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_FIFO_SET(void)
{
    if( tx_length == 0 )
    {
        do
        {
            Set_TSPI0_DR_value( tx_buf[tx_length] );
            tx_length++;
        
        } while( Get_TSPI0_TLVL_value() != ( TX_FILL_LEVEL + 1) );
        
        NVIC_EnableIRQ( INTSC0TX_IRQn );
        NVIC_EnableIRQ( INTSC0ERR_IRQn );

        enable_TSPI0_TRXE();

        proc = PROC_FIFO_TRANS;
    }
    else
    {
        tx_length = 0;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_FIFO_FINISHED.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_FIFO_FINISHED(void)
{
    if( Get_param_TSPI0_TFEMP() == FLAG_DETECT )
    {
        if( Get_param_TSPI0_TXRUN() == FLAG_NOT_DETECT )
        {
            disable_TSPI0_TRXE();

            NVIC_DisableIRQ( INTSC0TX_IRQn );
            NVIC_DisableIRQ( INTSC0ERR_IRQn );

            GPIO_LED1_FLIP();

            do
            {
                wait(10000);
                if( Get_PSW1_input() == BSP_PSW_OFF )
                {
                    sw_cnt++;
                }
                else
                {
                    sw_cnt = 0;
                }
            } while( sw_cnt < 10 );
            sw_cnt = 0;

            NVIC_ClearPendingIRQ( INT17_IRQn );
            Clear_flag_INTNCLR_IMC061();

            proc = PROC_FIFO_START;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_FIFO_ERROR.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_FIFO_ERROR(void)
{
    disable_TSPI0_TRXE();
    
    NVIC_DisableIRQ( INTSC0TX_IRQn );
    
    NVIC_DisableIRQ( INTSC0ERR_IRQn );
    
    tx_length = 0;
    
    proc = PROC_FIFO_START;
    
    GPIO_LED1_OFF();
    GPIO_LED3_ON();
}

/*--------------------------------------------------*/
/** 
  * @brief  INTSC0TX IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   
  */
/*--------------------------------------------------*/
void INTSC0TX_IRQHandler(void)
{
    if( proc == PROC_FIFO_TRANS )
    {
        if( Get_param_TSPI0_INTTXWF() == FLAG_DETECT )
        {
            Clear_flag_TSPI0_INTTXWF();
            
            Set_TSPI0_DR_value( tx_buf[tx_length] );
            tx_length++;
            
            if( tx_length >= DATA_LENGTH )
            {
                tx_length = 0;
                proc = PROC_FIFO_FINISHED;
            }
        }
        else
        {
            proc = PROC_FIFO_ERROR;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  INTSC0ERR IRQ Handler
  * @param  -
  * @return -
  * @retval -
  * @note   
  */
/*--------------------------------------------------*/
void INTSC0ERR_IRQHandler(void)
{
    Clear_allflag_TSPI0_ERR();
    proc = PROC_FIFO_ERROR;
}

#else
/* DMA_MODE ================================================= */
/*--------------------------------------------------*/
/** 
  * @brief  _PROC_DMAC_START.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_DMAC_START(void)
{
    DMAC_A_enable_fsys();
    DMA_Config();
    NVIC_EnableIRQ( INTDMAATC_IRQn );
    NVIC_EnableIRQ( INTDMAAERR_IRQn );

    INT17a_enable_IMC061();
    NVIC_EnableIRQ( INT17_IRQn );
    
    Set_param_TSPI0_TFEMPCLR();
    
    proc = PROC_DMAC_IDLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_DMAC_SET.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_DMAC_SET(void)
{
    
    enable_TSPI0_TRXE();
    
    if( TSPI_DMA_Trans( DATA_LENGTH, tx_buf) )
    {
        proc = PROC_DMAC_FINISHED;
    }
    else
    {
        proc = PROC_DMAC_ERROR;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_DMAC_FINISHED.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_DMAC_FINISHED(void)
{
    disable_TSPI0_TRXE();
    NVIC_DisableIRQ( INTDMAATC_IRQn );
    NVIC_DisableIRQ( INTDMAAERR_IRQn );
    
    GPIO_LED1_FLIP();

    do
    {
        wait(10000);
        if( Get_PSW1_input() == BSP_PSW_OFF )
        {
            sw_cnt++;
        }
        else
        {
            sw_cnt = 0;
        }
    } while( sw_cnt < 10 );
    sw_cnt = 0;

    NVIC_ClearPendingIRQ( INT17_IRQn );
    Clear_flag_INTNCLR_IMC061();

    proc = PROC_DMAC_START;
}

/*--------------------------------------------------*/
/** 
  * @brief  _PROC_DMAC_ERROR.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void _PROC_DMAC_ERROR(void)
{
    disable_TSPI0_TRXE();
    
    NVIC_DisableIRQ( INTDMAATC_IRQn );
    NVIC_DisableIRQ( INTDMAAERR_IRQn );
    
    proc = PROC_DMAC_START;
    
    GPIO_LED1_OFF();
    GPIO_LED3_ON();
}
#endif

/**
 *  @}
 */ /* End of group TSPI_MasterTrans_Exported_functions */

/**
 *  @}
 */ /* End of group TSPI_MasterTrans */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
